/*
 * Copyright 2012-2019 Intel Corporation
 *
 * This software and the related documents are Intel copyrighted materials, and
 * your use of them is governed by the express license under which they were
 * provided to you (License). Unless the License provides otherwise, you may
 * not use, modify, copy, publish, distribute, disclose, or transmit this
 * software or the related documents without Intel's prior written permission.
 *
 * This software and the related documents are provided as is, with no express
 * or implied warranties, other than those that are expressly stated in the
 * License.
 */

#ifndef __ANALYZE_EXTENSION_CLUSTER_DOT_H
#define __ANALYZE_EXTENSION_CLUSTER_DOT_H


#include <iostream>
#include <map>
#include <unordered_map>
#include <string>
#include <vector>

#include <config/node.h>


/*
 * The cluster namespace contains structs which represent the hardware and
 * software configuration of the cluster.
 *
 * TODO: Represent this data in a class hierarchy.
 */

namespace NsCluster {
    struct Node : public clck::Node {
        Node();
        Node(const clck::Node& n);

        std::string architecture;
        unsigned long uptime = 0;
        unsigned long last_reboot_time = 0;
        void *instance = nullptr;
        std::string instance_name;
    };
    typedef std::vector<Node> Nodes;


    struct Cluster {
        Nodes nodes;
        std::map<std::string,int> hostname_map;
        std::map<std::string,int>::const_iterator
            find(const std::string &hostname) const;
        //! begin const correct
        Nodes::const_iterator begin() const;
        //! end const correct
        Nodes::const_iterator end() const;
        operator const Nodes& () const;
    };
};


std::ostream& operator <<(std::ostream& s, const NsCluster::Node& a);

#endif //#ifndef __ANALYZE_EXTENSION_CLUSTER_DOT_H
