/*
 * Copyright 2012-2019 Intel Corporation
 *
 * This software and the related documents are Intel copyrighted materials, and
 * your use of them is governed by the express license under which they were
 * provided to you (License). Unless the License provides otherwise, you may
 * not use, modify, copy, publish, distribute, disclose, or transmit this
 * software or the related documents without Intel's prior written permission.
 *
 * This software and the related documents are provided as is, with no express
 * or implied warranties, other than those that are expressly stated in the
 * License.
 */

#ifndef __ANALYZE_EXTENSION_EXTENSION_DOT_H__
#define __ANALYZE_EXTENSION_EXTENSION_DOT_H__

#include <config/analyzer_config.h>
#include <config/templates.h>
#include <config/solution.h>
#include <generic_lib/nondeepcopyable_type.h>
#include <util/data_pt.h>
#include <util/clips_wrap.h>
#include <util/clips_wrap_internal.h>

#include "cluster.h"
#include "transform.h"

/**
 * TranformData is intended to pprovided a parent class for the classes
 * that mirror clips rules modules.  One TranformData subclass is intended
 * to be a similar concept to a clck2 module.
 */

class Extension : public Transform {

    public:
        Extension(void *clips_env);

        void set_cluster(const NsCluster::Cluster *cluster);

        void set_provider_rows(std::shared_ptr<clck::ProviderRows>& sp_m_provider_rows);

        // Pre-check
        virtual bool precheck(std::shared_ptr<clck::MissingData>& sp_m_missing_data);

        void set_expected_data(std::shared_ptr<std::map<std::string,
                               clck::Solution>> &expected_data);

        void set_CLIPS_type_map(std::shared_ptr<clck::templates::CLIPS_MAP_TYPE> &CLIPS_type_map);

        void set_CLIPS_data_type_map(std::shared_ptr<clck::templates::CLIPS_MAP_TYPE> &CLIPS_data_type_map);

        void set_conditional_variables(clck::Solution &s,
                clck::templates::Conditional_Type_vec &conditional_v,
                clck::templates::Unconditional_Type &unconditional_v);

        void update_header_route(std::map<std::string, DataPt> &header_route,
                clck::templates::Conditional_Type_vec &conditional_v,
                clck::templates::Unconditional_Type &unconditional_v,
                clck::templates::CLIPS_MAP_TYPE &type_map,
                clck::templates::CLIPS_MAP_TYPE &data_type_map);

        virtual bool sanitize_values(clck::templates::slot_map_type &slots,
                                     const clck::templates::Conditional_Type_vec& conditions = {}) const;


        void set_clips_classes(const std::set<std::string> &clips_list);

        const std::set<std::string> get_clips_classes() const ;

    protected:
        const NsCluster::Cluster *cluster;
        std::shared_ptr<clck::ProviderRows> sp_m_provider_rows{};
        std::shared_ptr<std::map<std::string, clck::Solution>> expected_data = nullptr;
        std::shared_ptr<clck::templates::CLIPS_MAP_TYPE> CLIPS_type_map = nullptr;
        std::shared_ptr<clck::templates::CLIPS_MAP_TYPE> CLIPS_data_type_map = nullptr;
        std::set<std::string> clips_classes = {};

    private:
        typedef Transform super;
        clck::templates::CLIPS_MAP_TYPE dataPt_map;

};


typedef Extension *CreateExtension(void *clips_env);

struct ExtensionData : public clck::util::NonDeepCopyable<ExtensionData> {
    void *libhandle = nullptr;
    CreateExtension *create = nullptr;;
    Extension *extension = nullptr;;
    ~ExtensionData();
};


#endif //#ifndef __ANALYZE_EXTENSION_TRANSFORM_DATA_DOT_H__
