/* 
 * Copyright 2012-2019 Intel Corporation
 *
 * This software and the related documents are Intel copyrighted materials, and
 * your use of them is governed by the express license under which they were
 * provided to you (License). Unless the License provides otherwise, you may
 * not use, modify, copy, publish, distribute, disclose, or transmit this
 * software or the related documents without Intel's prior written permission.
 *
 * This software and the related documents are provided as is, with no express
 * or implied warranties, other than those that are expressly stated in the
 * License.
 */

#ifndef __NETWORK_DEVICE_DOT_H__
#define __NETWORK_DEVICE_DOT_H__

#include "extension.h"


class NetworkDevice : public Extension {

 public:
    const static std::string intel_vendor_id;
    const static std::string mellanox_vendor_id;
    const static std::string qlogic_vendor_id;

    struct Device {
        Device();
        std::string name;
        std::string devpath;

        std::vector<std::string> ip4_addrs;
        std::vector<std::string> ip6_addrs;
        std::string mac;
        std::string mac_fmt;

        std::vector<std::string> dapl_providers;
        std::vector<std::string> ofa_adapters;
        std::vector<std::string> tmi_providers;
        std::vector<std::string> ofi_providers;
        std::string link_layer;
        std::string port_guid;
        std::string driver;
        std::string driver_version;
        int rx_usecs;
    };
    typedef std::vector<Device> Devices;


    struct Adapter {
        std::string str_verbose();
        std::string pci_addr;
        std::string vendor_id;
        std::string model;
        std::string model_id;
        Devices devices;
        std::string instance_name;
        std::string driver;
        std::vector<std::string> kernel_mods;
    };
    typedef std::vector<Adapter> Adapters;

    NetworkDevice(void *clips_env);
    virtual bool parse();

    static void get_net_instance_name(void *clips_env, Adapter& adapter,
                                      std::string hostname);

    static bool net_udevadm_parse(const std::string& hostname,
                                  Devices& devices);

    static size_t get_net_device_idx(const Devices& devices,
                                  const std::string& name);

    static std::string get_link_str(const std::string& s);

    static bool net_ipaddr_parse(const std::string& hostname,
                                 Devices& devices);

    static size_t get_net_adapter_idx(const Adapters& adapters,
                                   const std::string& pci_addr);

    static size_t net_lspci_parse_adapter(const std::vector<std::string>& lines,
                                          size_t line_idx,
                                          Adapter& adapter);

    static bool net_lspci_parse(const std::string& hostname,
                                Adapters& adapters, Devices& devices);

    static bool mac_eq_guid(const std::string& mac, std::string guid);

    static bool net_ibstat_parse(const std::string& hostname,
                                 Devices& devices);

    static bool net_datconf_parse(
                                  const std::string& hostname,
                                  Adapters& adapters);

    static bool net_tmiconf_parse(
                                  const std::string& hostname,
                                  Adapters& adapters);

    static bool net_ofi_setup(Adapters& adapters);

    static bool net_ethtool_parse(const std::string& hostname,
                                  Adapter& adapter);

    static bool net_ethtool_coalesce_parse(
                                           const std::string& hostname,
                                           Adapter& adapter);

    static bool net_ofedinfo_parse(
                                   const std::string& hostname,
                                   Adapter& adapter);

    static bool net_driver(const std::string& hostname,
                           Adapters& adapters);

    bool get_dapl_expected_perf(const std::string& hostname,
                                const std::string& dapl_provider,
                                double& bandwidth, double& latency);

    bool get_ofa_expected_perf(const std::string& hostname,
                               const std::string& ofa_adapter,
                               double& bandwidth, double& latency);

    bool get_tmi_expected_perf(const std::string& hostname,
                               const std::string& tmi_provider,
                               double& bandwidth, double& latency);


    bool get_ofi_expected_perf(const std::string& hostname,
                               const std::string& ofi_provider,
                               double& bandwidth, double& latency);

    const std::map<std::string, Adapters>& get_node_map();

 protected:
    std::map<std::string, Adapters> node_map;

 private:
    typedef Extension super;

};


std::ostream& operator <<(std::ostream& s, const NetworkDevice::Device& a);
std::ostream& operator <<(std::ostream& s, const NetworkDevice::Adapter& a);

#endif //#ifndef __NETWORK_DEVICE_DOT_H__
