/*
 * Copyright 2012-2019 Intel Corporation
 *
 * This software and the related documents are Intel copyrighted materials, and
 * your use of them is governed by the express license under which they were
 * provided to you (License). Unless the License provides otherwise, you may
 * not use, modify, copy, publish, distribute, disclose, or transmit this
 * software or the related documents without Intel's prior written permission.
 *
 * This software and the related documents are provided as is, with no express
 * or implied warranties, other than those that are expressly stated in the
 * License.
 */

#ifndef __ANALYZE_EXTENSION_TRANSFORM_DOT_H__
#define __ANALYZE_EXTENSION_TRANSFORM_DOT_H__

#include <clips/clips.h>
#include <datastore/datastore.h>
#include <generic_lib/nondeepcopyable_type.h>
#include <util/data_pt.h>

/**
 * Transformers are generic data processors
 * Their functionality is as follows:
 *   Parse data from the data providers
 *   Filter the data
 *   Populate the corresponding clips objects/instances
 */
class Transform : public clck::util::NonDeepCopyable<Transform> {

    public:

        enum ROUTES {
            ROUTE_NONE      = 0,
            ROUTE_CSV       = 1,
            ROUTE_CLIPS_STR = 2,
            ROUTE_CLIPS_API = 4,
            ROUTE_CLIPS_API_MAP = 5
        };

        Transform(void *clips_env);
        virtual ~Transform() = default;

        std::string get_name();

        ///read data from the input source
        virtual bool parse();

        ///read data from the input source for comparison
        virtual bool compare_snapshots();

        //set output file stream
        void set_csv_output_stream(std::ostream *output_stream);

        //clips instance name
        void set_name(const std::string& name);

        //clips instance name
        void set_clips_class_name(const std::string& name);

        //column header for data points
        void set_header(const DataPtsHeader& h);

        int get_route_mask();
        void set_route_mask(int mask);
        bool is_route_active(int route);
        std::map<std::string, std::vector<std::string>> get_csv_map();

        //output a row of data points
        virtual void route(const DataPts& d);
        virtual void route(const std::map<std::string,DataPt> &header_route);
        void route_csv(const DataPts& d);
        void route_clips_str(const DataPts &d);
        void route_clips_str(const std::map<std::string,DataPt> &header_route);
        void route_clips_api(const DataPts &d);
        void route_clips_api(const std::map<std::string, DataPt> &header_route);

    protected:

        std::string name;
        void *clips_env;

        std::string clips_class_name;
        std::vector<std::string> data_header;

        std::ostream *csv_ostream;
        std::map<std::string, std::vector<std::string>> csv_map;

        int route_mask;
};


#endif ///#ifndef __ANALYZE_EXTENSION_TRANSFORM_DOT_H__
