/*
 * Copyright 2018-2019 Intel Corporation
 *
 * This software and the related documents are Intel copyrighted materials, and
 * your use of them is governed by the express license under which they were
 * provided to you (License). Unless the License provides otherwise, you may
 * not use, modify, copy, publish, distribute, disclose, or transmit this
 * software or the related documents without Intel's prior written permission.
 *
 * This software and the related documents are provided as is, with no express
 * or implied warranties, other than those that are expressly stated in the
 * License.
 */

#ifndef _CLCK_CLCK_H
#define _CLCK_CLCK_H

#include <map>
#include <memory>
#include <string>
#include <vector>

namespace clck{
  class AnalysisOutput;
}

namespace clckapi{
  class ClckApi;

  class Clck final {
  public:
    Clck();
    ~Clck();

    // execution functions
    bool Collect();
    bool CollectAndAnalyze(std::vector<std::shared_ptr<clck::AnalysisOutput>>& out);
    bool Analyze(std::vector<std::shared_ptr<clck::AnalysisOutput>>& out);
    bool RunPostprocessor(const std::string& outputFile = "");
    std::string Version();

    // configuration options that are consistent with cmdline options
    // returns true for successful.
    bool SetConfigFile(const std::string& path);
    bool SetDatabaseFile(const std::string& path);
    bool SetFailLevel(const std::string& severity);
    bool SetFrameworkDefinitions(const std::vector<std::string>& fwd_paths);
    bool SetIgnoreSubclusters(const bool& val);
    bool SetLogLevel(const std::string& log_level);
    bool SetLogFile(const std::string& path);
    bool SetNodeFile(const std::string& path);
    bool SetPermutations(const int& number_of_permutations);

    // configuration options that are api only (not commandline)
    // returns true for successful.
    bool SetNodes(const std::vector<std::string>& nodes);
    bool SetSuppressions(const std::vector<std::map<std::string,
                         std::vector<std::string>>>& suppressions);
    bool SetConfigOption(const std::string& tag, const std::string& value);

  private:
    ClckApi* clckapi;
  };
} // namespace clckapi

#endif // _CLCK_CLCK_H
