/*
 * Copyright 2017-2019 Intel Corporation
 *
 * This software and the related documents are Intel copyrighted materials, and
 * your use of them is governed by the express license under which they were
 * provided to you (License). Unless the License provides otherwise, you may
 * not use, modify, copy, publish, distribute, disclose, or transmit this
 * software or the related documents without Intel's prior written permission.
 *
 * This software and the related documents are provided as is, with no express
 * or implied warranties, other than those that are expressly stated in the
 * License.
 */

#ifndef extension_h
#define extension_h

#include <set>
#include <string>
#include <generic_lib/nondeepcopyable_type.h>

struct CollectExtension : public clck::util::NonDeepCopyable<CollectExtension> {
    virtual ~CollectExtension() = default;

    // Pre-check extension functionality and node connectivity.
    virtual bool precheck(const std::set<std::string> &nodes) = 0;

    // Run the command on the nodes.
    virtual bool run(const std::string &command, const std::set<std::string> &nodes) = 0;

    // A token that can be used inside the command string that the extension will
    // replace with the hostname the command is running on.
    virtual std::string hostname_subst_token() const = 0;

    // The stdout generated by the last run() call.
    virtual const std::string& stdout() const = 0;

    // The stderr generated by the last run() call.
    virtual const std::string& stderr() const = 0;

    // The exit status generated by the last run() call.
    virtual int exit_status() const = 0;

    // The terminate method to handle cleaning up.
    virtual int terminate() = 0;

    // Default constructor
    CollectExtension() = default;
};

#endif
