/*
 * Copyright 2017-2019 Intel Corporation
 *
 * This software and the related documents are Intel copyrighted materials, and
 * your use of them is governed by the express license under which they were
 * provided to you (License). Unless the License provides otherwise, you may
 * not use, modify, copy, publish, distribute, disclose, or transmit this
 * software or the related documents without Intel's prior written permission.
 *
 * This software and the related documents are provided as is, with no express
 * or implied warranties, other than those that are expressly stated in the
 * License.
 */

#ifndef _CLCK_ANALYSIS_OUTPUT_H
#define _CLCK_ANALYSIS_OUTPUT_H

#include <string>
#include <vector>
#include <set>

namespace clck {

    // @brief AnalyzeOutput is the basic analysis unit.  It may be either a
    //        sign (i.e., observation) or a diagnosis (i.e., root
    //        cause).
    class AnalysisOutput {
    public:
        int confidence                 = 0;     //**< Confidence percentage (0 - 100)
        std::string id                 = "";    //**< Message catalog id
        std::string msg                = "";    //**< Expanded message string
        std::vector<std::string> nodes = {};    //**< List of nodes
        std::string remedy             = "";    //**< Expanded remedy string
        int severity                   = 0;     //**< Severity percentage (0 - 100)
        bool suppressed                = false; //**< True if the diagnosis / sign is
                                                //   suppressed, false otherwise
        std::set<int> rowid;                    //**< DB rows that provide the raw data
                                                //   upon which the sign/diagnosis is based
    };

}; // clck

#endif //_CLCK_ANALYSIS_OUTPUT_H


