/*
 * Copyright 2012-2019 Intel Corporation
 *
 * This software and the related documents are Intel copyrighted materials, and
 * your use of them is governed by the express license under which they were
 * provided to you (License). Unless the License provides otherwise, you may
 * not use, modify, copy, publish, distribute, disclose, or transmit this
 * software or the related documents without Intel's prior written permission.
 *
 * This software and the related documents are provided as is, with no express
 * or implied warranties, other than those that are expressly stated in the
 * License.
 */

#ifndef __UTIL_PROVIDER_H__
#define __UTIL_PROVIDER_H__


#include <limits.h>
#include <map>
#include <string>

namespace clck {
namespace provider {

    enum encoding { ENCODING_NONE   = 0,
                    ENCODING_BASE64 = 1,
                    ENCODING_RAW    = 2 };

    /* encoding XML configuration tag value -> enum equivalent */
    static const std::map<std::string, clck::provider::encoding> encoding_map = {
      {"none", clck::provider::ENCODING_NONE},
      {"base64", clck::provider::ENCODING_BASE64},
      {"raw", clck::provider::ENCODING_RAW},
    };

    const ssize_t MAX_PROVIDER_NAME = 64;

    /* This header gets written in to the final output file. Fields from
     * this header are used in the database.  Basic C data types must be
     * used until a time when the struct is actually serialized for I/O.
     */
    struct header {
        uint8_t api = 1; /* Header API version . Currently 1 */
        int ret = 0;
        char name[MAX_PROVIDER_NAME] = ""; /* Obtained from XML file */
        enum encoding encoding = clck::provider::encoding::ENCODING_NONE;
        timeval timestamp = {}; /* Calculated at the time of running */
        double duration = 0.0; /* Calculated after running */
        char hostname[_POSIX_HOST_NAME_MAX] = "";
        char nodename[_POSIX_HOST_NAME_MAX] = "";
        char username[_POSIX_LOGIN_NAME_MAX] = "";
        ssize_t nodes_csv_size = 0; /* Used only by cluster providers */
        uint16_t num_nodes = 1; /* Used only by cluster providers */
        ssize_t stdout_size = 0; /* Will be computed after running command */
        ssize_t stderr_size = 0; /* Will be computed after running command */
        char OptionID[60] = ""; /* User configured or "default" */
        int version = 1; /* User configured or defaulted to 1 */
        timeval unique_timestamp = {}; /*unique timestamp identifier per run*/
    };


}; //namespace provider
}; //namespace clck

#endif //__UTIL_PROVIDER_H__
