/*
 * Copyright 2012-2019 Intel Corporation
 *
 * This software and the related documents are Intel copyrighted materials, and
 * your use of them is governed by the express license under which they were
 * provided to you (License). Unless the License provides otherwise, you may
 * not use, modify, copy, publish, distribute, disclose, or transmit this
 * software or the related documents without Intel's prior written permission.
 *
 * This software and the related documents are provided as is, with no express
 * or implied warranties, other than those that are expressly stated in the
 * License.
 */

#ifndef __TEMPLATES_DOT_H__
#define __TEMPLATES_DOT_H__

#include<algorithm>
#include<map>
#include<set>
#include<vector>

namespace clck {
  namespace templates{

    //first elem in pair is slot name, second is expected value
    using Slot_expected_pair = std::pair<std::string, std::string>;
    //@first is the vector of slots
    //@second is the reports catalog msg-id
    //using Report_value_pair = std::pair< std::vector<Slot_expected_pair>, std::string>;
    using Report_value_pair = std::pair<std::vector<std::string>,std::string>;


    //ex KB_CLASS Memory
    //string(slot)   --> attr type --> attr        --> value
    //physical-total --> node-role      --> control     - 900GB
    //                             --> application - 5GB
    //                             --> storage     - 1TB
    using Conditional_value_type = std::tuple<std::string,std::string,std::string>;
    using Conditional_Type  = std::pair<std::string, Conditional_value_type>;
    using Conditional_Type_vec = std::vector<Conditional_Type>;
    using Unconditional_Type = std::vector<Slot_expected_pair>;

    //! @brief: struct to pair values with constraints
    struct Reports_kb_t {
        //! msg_id, values pair
        Report_value_pair values = {};
        //! constraints
        Conditional_Type_vec evaluation_conditions = {};
    };

    //! struct to capture multiple reports per kb
    struct Reports_t {
        std::vector<Reports_kb_t> reports;
    };

    //@key =  kb Class mapped to slots, msg_id
    using Report_values_type =  std::map<std::string, Reports_t>;

    using Reports_Type = Report_values_type;

    using Reports_Map_Type = std::map<std::string, std::set<std::string>>;
    using Class_Reports_Map_Type = std::map<std::string, Reports_Map_Type>;

    //map of particular slot to  explicit attribute-value
    //ex memory: node-role - pair<control-900GB>
    //ex memory: node-role - application<application-500GB>
    using Attribute_Type = std::map<std::string, std::vector<Slot_expected_pair>>;
    //map of slot to attribute type
    //ex memory: physical-total to node-role
    using Attributes_Map_Type = std::map<std::string, Attribute_Type>;

    enum DataPt_TYPES {
         TYPE_UNKNOWN = 0,
         TYPE_LONG    = 1,
         TYPE_DOUBLE  = 2,
         TYPE_STRING  = 3,
         TYPE_MULTI   = 4
    };

    using CLIPS_MAP_TYPE = std::map<std::string, DataPt_TYPES>;
    using KB_CONFIG_TYPE = std::vector<std::string>;

    //! returns expected constraint value
    inline std::string get_constraint_name(const Conditional_Type &val) {
        return val.first;
    }

    //! returns expected constraint value
    inline std::string get_constraint(const Conditional_Type &val) {
        return std::get<0>(val.second);
    }

    //! returns field on which constrain is applied
    inline std::string get_constraint_field(const Conditional_Type &val) {
        return std::get<1>(val.second);
    }

    //! returns field on which constrain is applied
    inline std::string get_constraint_value(const Conditional_Type &val) {
        return std::get<2>(val.second);
    }

     using slot_map_type = std::map<std::string,std::vector<std::string>>;
  };
};
#endif //__TEMPLATES_DOT_H__
