/*
 * Copyright 2018 Intel Corporation
 *
 * This software and the related documents are Intel copyrighted materials, and
 * your use of them is governed by the express license under which they were
 * provided to you (License). Unless the License provides otherwise, you may
 * not use, modify, copy, publish, distribute, disclose, or transmit this
 * software or the related documents without Intel's prior written permission.
 *
 * This software and the related documents are provided as is, with no express
 * or implied warranties, other than those that are expressly stated in the
 * License.
 */

#ifndef __CLCK__IMMOVABLE_CLASS_TYPE__
#define __CLCK__IMMOVABLE_CLASS_TYPE__

#include <memory>

namespace clck
{
namespace util
{
//! Base class for disabling trivial copy/move
template<typename T>
class NonDeepCopyable
{
  public:
  //! Disable all the copies/moves
  NonDeepCopyable& operator=(const NonDeepCopyable&) = delete;
  NonDeepCopyable& operator=(NonDeepCopyable&&) = delete;
  NonDeepCopyable(const NonDeepCopyable&) = delete;
  NonDeepCopyable(NonDeepCopyable&&) = delete;

  // The extension must implement a create() function with this signature.
  using createFunction = T* (*)();

  // The extension must implement a destroy() function with this signature.
  using destroyFunction = void (*)(T*);

  //! Export type for NonDeepCopyable objects
  using nonowned_export_type = const std::weak_ptr<T>;

  protected:
  NonDeepCopyable() = default;
  virtual ~NonDeepCopyable() = default;
};
}   // util
}   // clck
#endif
