/*
 * Copyright 2018-2019 Intel Corporation
 *
 * This software and the related documents are Intel copyrighted materials, and
 * your use of them is governed by the express license under which they were
 * provided to you (License). Unless the License provides otherwise, you may
 * not use, modify, copy, publish, distribute, disclose, or transmit this
 * software or the related documents without Intel's prior written permission.
 *
 * This software and the related documents are provided as is, with no express
 * or implied warranties, other than those that are expressly stated in the
 * License.
 */

#ifndef __CLCK3_CLIPS_WRAP_INTERNAL__
#define __CLCK3_CLIPS_WRAP_INTERNAL__

#include <cpil2.i18n.hpp>
#include "clips_wrap.h"
#include <generic_lib/log.h>


bool get_slot_multi_generic(void *clips_env, void *inst, const std::string& slot,
        DATA_OBJECT& d, std::vector<std::string>& values,
        std::vector<cpil2::variant> &args);

bool generic_get_slot_values(void *clips_env, void *inst, const std::string& slot, DATA_OBJECT &data_obj, std::vector<std::string>& values, std::vector<cpil2::variant> &args);

template <
 typename T = std::vector<std::string>,
 typename U = std::vector<cpil2::variant>
>
int query_kb_slots(void *clips_env, void* instance, const std::string slot, const std::string kb_class,
                   typename std::remove_reference<T>::type &values,
                   typename std::remove_reference<U>::type &args) {
    if(clips_env == nullptr || instance == nullptr || slot.empty() || kb_class.empty()) {
       return EXIT_FAILURE;
    }
    void *clips_class = get_clips_class(clips_env, kb_class);
    bool is_single;
    int data_type;
    //getting type to decide get value function
    bool found_slot = get_slot_type(clips_env, clips_class, slot , is_single, data_type);
    if(!found_slot){
       log_warn("slot: " << slot << " does not exist for "<<kb_class);
       return EXIT_FAILURE;
    }

    DATA_OBJECT ts{};
    if(is_single){
       //gets the slot value based on type for single slots
       generic_get_slot_values(clips_env, instance, slot, ts, values, args);
    }
    else{
       //get the multiset values converted to string vec
       get_slot_multi_generic(clips_env, instance, slot, ts, values, args);
    }
    return EXIT_SUCCESS;
}


#endif //#ifndef __CLCK3_CLIPS_WRAP__
