/*
 * Copyright 2012-2019 Intel Corporation
 *
 * This software and the related documents are Intel copyrighted materials, and
 * your use of them is governed by the express license under which they were
 * provided to you (License). Unless the License provides otherwise, you may
 * not use, modify, copy, publish, distribute, disclose, or transmit this
 * software or the related documents without Intel's prior written permission.
 *
 * This software and the related documents are provided as is, with no express
 * or implied warranties, other than those that are expressly stated in the
 * License.
 */

/* TODO rewrite using templates
   The DpMulti interface is more difficult to use that it should be. */


#ifndef __DATA_PTS_DOT_H__
#define __DATA_PTS_DOT_H__

#include <istream>
#include <string>
#include <vector>
#include <sstream>
#include <memory>
#include <config/templates.h>

/// data point header
typedef std::vector<std::string> DataPtsHeader;


/// base class
struct DataPt {

    DataPt();
    DataPt(const long& d);
    DataPt(const int& d);
    DataPt(const double& d);
    DataPt(const std::string& d);
    DataPt(const char *d);

    virtual ~DataPt();

    void set_long(const long& v);
    void set_double(const double& v);
    void set_string(const std::string& v);

    int type;
    long val_l{0};
    double val_d{0.0};
    std::string val_s;

    bool is_type_single() const;

    std::vector<std::shared_ptr<DataPt>> vals_dp;
};
std::ostream& operator <<(std::ostream& s, const DataPt& d);

typedef std::vector<DataPt> DataPts;


/// single data point

struct DpLong : public DataPt {

    DpLong();
    DpLong(const long& d);
    DpLong(const int& d);
    DpLong(const DataPt *dp);
};
void operator >>(std::istream& s, DpLong& d);


struct DpDouble : public DataPt {

    DpDouble();
    DpDouble(const double& d);
    DpDouble(const DataPt *dp);
};
void operator >>(std::istream& s, DpDouble& d);

struct DpString : public DataPt {

    DpString();
    DpString(const std::string& d);
    DpString(const DataPt *dp);
    DpString(const char *d);
};
void operator >>(std::istream& s, DpString& d);
bool operator ==(const DataPt& a, const DataPt& b);


/// multi data point

struct DpMulti : public DataPt {

    DpMulti();
    DpMulti(const DataPt *dp);

    void append(const DataPt *dp);
    void append(const DataPt& dp);
};


#endif //#ifndef __DATA_PTS_DOT_H__
