; Copyright 2018-2019 Intel Corporation
;
; This software and the related documents are Intel copyrighted materials, and
; your use of them is governed by the express license under which they were
; provided to you (License). Unless the License provides otherwise, you may
; not use, modify, copy, publish, distribute, disclose, or transmit this
; software or the related documents without Intel's prior written permission.
;
; This software and the related documents are provided as is, with no express
; or implied warranties, other than those that are expressly stated in the
; License.

(defclass DEVICES
  "This class represents devices found by lspci_verbose"
  (is-a BASE_NODE BASE_TIMESTAMP MULTISET DATABASE)
  (role abstract)

  (slot err (type STRING) (default ""))
  ; device index
  (multislot device_idxs (type NUMBER) (visibility public))
  (multislot index (type NUMBER) (visibility public))
  ; PCIe device type
  (multislot device_types (type SYMBOL) (visibility public))
  ; PCIe device id
  (multislot device_ids (type NUMBER) (visibility public))
  ; PCIe device vendor
  (multislot device_vendors (type NUMBER) (visibility public))
)

(defmessage-handler DEVICES get-device-id-by-idx(?idx)
  (return (nth$ ?idx ?self:device_ids)))

(defmessage-handler DEVICES get-device-vendor-by-idx(?idx)
  (return (nth$ ?idx ?self:device_vendors)))

(defmessage-handler DEVICES get-device-type-by-idx(?idx)
  (return (nth$ ?idx ?self:device_types)))

(defclass LSPCI_DEVICES
  "This class represents devices found in the sys_fs directories"
  (is-a DEVICES)
  (role concrete)
  (pattern-match reactive)

  ; PCIe linkcap width
  (multislot lnkcap_width (type SYMBOL) (visibility public))
)

(defmessage-handler LSPCI_DEVICES get-device-width-by-idx(?idx)
  (return (nth$ ?idx ?self:lnkcap_width)))

(defclass SYS_DEVICES
  "This class represents devices found in the sys_fs directories"
  (is-a DEVICES)
  (role concrete)
  (pattern-match reactive)
 
  ; device status
  (multislot device_state (type STRING) (visibility public))
  ; device size
  (multislot device_size (type INTEGER) (visibility public))
  ; device speed
  (multislot device_speed (type INTEGER) (visibility public))
  ; device label
  (multislot device_label (type STRING) (visibility public))
  ; device model
  (multislot device_model (type STRING) (visibility public))
  ; device fw
  (multislot device_fw (type STRING) (visibility public))
)

(defmessage-handler SYS_DEVICES get-device-state-by-idx(?idx)
  (return (nth$ ?idx ?self:device_state)))

(defmessage-handler SYS_DEVICES get-device-size-by-idx(?idx)
  (return (nth$ ?idx ?self:device_size)))

(defmessage-handler SYS_DEVICES get-device-speed-by-idx(?idx)
  (return (nth$ ?idx ?self:device_speed)))

(defmessage-handler SYS_DEVICES get-device-label-by-idx(?idx)
  (return (nth$ ?idx ?self:device_label)))

(defmessage-handler SYS_DEVICES get-device-model-by-idx(?idx)
  (return (nth$ ?idx ?self:device_model)))

(defmessage-handler SYS_DEVICES get-device-fw-by-idx(?idx)
  (return (nth$ ?idx ?self:device_fw)))

(defmessage-handler SYS_DEVICES init after ()
  ;"Add listed attributes to check for uniformity: firmware"
  (progn$ (?index ?self:device_idxs)
  (bind ?fw (send ?self get-device-fw-by-idx ?index))
  (bind ?type (send ?self get-device-type-by-idx ?index))
  (if (neq (str-compare "Unknown" ?fw) 0) then
    (send ?self add (send ?self uniformity-key ?type) ?fw)))
)

(defmessage-handler SYS_DEVICES parse-uniformity (?index)
  (bind ?type (send ?self get-device-type-by-idx ?index))
  (bind ?u-key (send ?self uniformity-key ?type))
  (bind ?fw (send ?self get-device-fw-by-idx ?index))
  (return (send ?self fraction ?u-key ?fw))
)

(defmessage-handler SYS_DEVICES uniformity-key (?type)
  (bind ?key (sym-cat (class ?self)
            + ?self:subcluster + ?type))
  (return ?key)
)

(defclass REPORT_DEVICE
  "Class to spit out device info for reports"
  (is-a BASE_NODE BASE_TIMESTAMP DATABASE)
  (role concrete)

  ; PCIe device type
  (slot device_type (type STRING) (visibility public))
  ; PCIe device id
  (slot device_id (type NUMBER) (visibility public))
  ; PCIe device label
  (slot device_label (type STRING) (visibility public))
  ; PCIe device vendor
  (slot device_vendor (type NUMBER) (visibility public))
  ; PCIe device firmware
  (slot device_fw (type STRING) (visibility public))

)
