 ; Copyright 2018-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass HARDWARE_SPEC
  "This class is used for hardware entries for lshw for disk-related items."
  (is-a BASE_NODE TIMESTAMP MULTISET DATABASE)
  (slot product (type STRING) (default "unknown"))
  (slot physid (type STRING) (default "unknown"))
  (slot firmware-version (type STRING) (default "unknown"))
  (slot description (type STRING) (default "unknown"))
  (slot logical-name (type STRING) (default "unknown"))
  (multislot capabilities (type STRING))
)

(defclass DISK
  "This class is used for disk hardware entries."
  (is-a HARDWARE_SPEC)
  (role concrete)
  (pattern-match reactive)
)

(defclass RAID
  "This class is used for raid hardware entries."
  (is-a HARDWARE_SPEC)
  (role concrete)
  (pattern-match reactive)
)

(defclass DISK_COUNT
  "This class is to count number of disks per node."
  (is-a BASE_NODE BASE_TIMESTAMP DATABASE)
  (slot disk-count (type INTEGER) (default 0))
  (role concrete)
  (pattern-match reactive)
)

(defmessage-handler HARDWARE_SPEC init after ()
  (send ?self add (send ?self uniformity-key product) ?self:product)
  (send ?self add (send ?self uniformity-key-product-based firmware-version)
      ?self:firmware-version)
)

(defmessage-handler HARDWARE_SPEC uniformity-key (?v)
  (bind ?role compute)
  (bind ?subcluster default)

  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?subcluster (send ?i get-subcluster))
    (if (member$ storage (send ?i get-role)) then
      (bind ?role storage)))
    
  (bind ?key (sym-cat (class ?self) + ?role + ?v))
  (return ?key)
)

(defmessage-handler HARDWARE_SPEC uniformity-key-product-based (?v)
  (bind ?role compute)
  (bind ?subcluster default)
    
  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?subcluster (send ?i get-subcluster))
    (if (member$ storage (send ?i get-role)) then
      (bind ?role storage)))

  (bind ?key (sym-cat (class ?self) + ?role + ?self:product + ?v))
  (return ?key)
)
