 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass ENVIRONMENT
  (is-a BASE_NODE BASE_TIMESTAMP MULTISET DATABASE)
  (role concrete)
  (pattern-match reactive)


  (multislot envblacklist (storage shared) (type STRING) (access read-only)
                          (default (create$ "HOST" "HOSTNAME" "OLDPWD"
                                    "SSH_CLIENT" "SSH_CONNECTION" "SHOST"
                                    "I_MPI_PIN_MAPPING" "PMI_RANK"
                                    "SLURMD_NODENAME" "SLURM_GTIDS"
                                    "SLURM_NODEID" "SLURM_PROCID"
                                    "SLURM_TASK_PID" "SLURM_TOPOLOGY_ADDR"
                                    "CLCK_TIMEOUT"
                                    "MPIR_CVAR_CH3_INTERFACE_HOSTNAME"
                                    "SSH2_CLIENT" "XDG_SESSION_ID" "^_$")))

  ; This slot represents the environment variable
  (slot variable (type SYMBOL) (default UNKNOWN))

  ; This slot represents the value of the environment variable.  In
  ; the case of PATH like variables, this is the trimmed value based
  ; on what was found on pure compute nodes For regular environment
  ; variables this is equal to the true-value
  (slot value (type STRING) (default ""))

  ; The unmodified value of the environment variable as seen in the
  ; database
  (slot true-value (type STRING) (default ""))

  ; This is set by the analyze extension. If the analyze extension sets it to TRUE,
  ; this particular variable will be checked for uniformity.  The
  ; analyze extension will decide based on node roles.
  (slot require-uniformity (type SYMBOL) (default UNKNOWN)))

(defmessage-handler ENVIRONMENT uniformity-key ()
  (bind ?role nil)
  (bind ?subcluster nil)
  (bind ?arch nil)
  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then 
    (bind ?i (nth$ 1 ?ins))
    (bind ?arch (send ?i get-architecture))
    (bind ?subcluster (send ?i get-subcluster)))

  (bind ?key (sym-cat (class ?self) + ?subcluster + ?arch + ?self:variable))
  (return ?key))

(defmessage-handler ENVIRONMENT init after ()
  (if (eq ?self:require-uniformity FALSE) then
    (return 0))

  (send ?self add (send ?self uniformity-key) ?self:value))
