 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass ETHERNET
  "Class for Ethernet devices"
  (is-a BASE_NODE BASE_TIMESTAMP MULTISET DATABASE)
  (role concrete)
  (pattern-match reactive)

  (slot driver (type STRING) (default ""))
  (slot firmware-version (type STRING) (default ""))
  (slot interface (type SYMBOL) (default nil))
  (slot mac (type STRING) (default ""))
  (slot mtu (type INTEGER) (default 0))
  (slot rx-usecs (type INTEGER) (default 65536))
  (slot state (type SYMBOL) (default nil))
  (slot version (type STRING) (default "")))

(defmessage-handler ETHERNET init after ()
  (send ?self add (send ?self uniformity-key driver) ?self:driver)
  (send ?self add (send ?self uniformity-key firmware-version)
        ?self:firmware-version)
  (send ?self add (send ?self uniformity-key mtu) ?self:mtu)
  (send ?self add (send ?self uniformity-key version) ?self:version))

(defmessage-handler ETHERNET uniformity-key (?v)
  (bind ?role nil)
  (bind ?subcluster nil)
  (bind ?arch nil)
  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?arch (send ?i get-architecture))
    (bind ?subcluster (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) + ?subcluster + ?role + ?arch +
                      ?self:interface + ?v))
  (return ?key))
