 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass HPL
  (is-a CLUSTER BASE_TIMESTAMP IMPI STATISTICS DATABASE)
  (role concrete)
  (pattern-match reactive)

  (slot gflops (type NUMBER) (default 0))
  (slot n (type INTEGER) (default 0))
  (slot nb (type INTEGER) (default 0))
  (slot p (type INTEGER) (default 0))
  (slot q (type INTEGER) (default 0))
  (slot residual (type SYMBOL) (allowed-values PASSED FAILED UNKNOWN)
        (default UNKNOWN))
  (slot rmax (type NUMBER) (default 0))
  (slot execution-time (type NUMBER) (default 0))
  (slot provider (type SYMBOL) (allowed-values CLUSTER PAIRWISE)
        (default PAIRWISE))
  (slot number-of-coprocessors (type INTEGER) (default 0))
  (slot OptionID (type SYMBOL)))

(defmessage-handler HPL statistics-key ()
  "Return a key that is unique for each combination of subcluster,
   role, option id, and fabric."
  (bind ?role1 nil)
  (bind ?role2 nil)
  (bind ?subcluster1 nil)
  (bind ?subcluster2 nil)

  (bind ?ins1 (find-instance ((?n NODE))
                             (eq ?n:node_id (nth$ 1 ?self:node_id))))
  (if (= (length ?ins1) 1) then
    (bind ?i (nth$ 1 ?ins1))
    (bind ?subcluster1 (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role1 compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role1 enhanced))))

  (bind ?ins2 (find-instance ((?n NODE))
                             (eq ?n:node_id (nth$ 2 ?self:node_id))))
  (if (= (length ?ins2) 1) then
    (bind ?i (nth$ 1 ?ins2))
    (bind ?subcluster2 (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role2 compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role2 enhanced))))

  (switch ?self:i_mpi_fabrics
          (case dapl then
                (bind ?subfabric ?self:i_mpi_dapl_provider))
          (case ofa then
                (bind ?subfabric ?self:i_mpi_ofa_adapter_name))
          (case ofi then
                (bind ?subfabric ?self:i_mpi_ofi_provider))
          (case shm then
                (bind ?subfabric nil))
          (case shm:dapl then
                (bind ?subfabric ?self:i_mpi_dapl_provider))
          (case shm:ofa then
                (bind ?subfabric ?self:i_mpi_ofa_adapter_name))
          (case shm:ofi then
                (bind ?subfabric ?self:i_mpi_ofi_provider))
          (case shm:tcp then
                (bind ?subfabric ?self:i_mpi_tcp_netmask))
          (case shm:tmi then
                (bind ?subfabric ?self:i_mpi_tmi_provider))
          (case tcp then
                (bind ?subfabric ?self:i_mpi_tcp_netmask))
          (case tmi then
                (bind ?subfabric ?self:i_mpi_tmi_provider))
          (default bind ?subfabric nil))

  (bind ?key (sym-cat (class ?self)
                      - ?subcluster1 - ?subcluster2 - ?role1 - ?role2
                      - ?self:i_mpi_fabrics - ?subfabric
                      - ?self:number-of-coprocessors - ?self:OptionID))
  (return ?key))

(defmessage-handler HPL init after()
  (if (eq ?self:provider PAIRWISE) then
    (if (eq ?self:residual PASSED) then
      (send ?self add (send ?self statistics-key) ?self:gflops))))

