 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass IMB_PINGPONG
  (is-a BASE_TIMESTAMP NODE_PAIR IMPI STATISTICS DATABASE)
  (role concrete)
  (pattern-match reactive)

  ; architecture of endpoints
  (multislot arch (type SYMBOL) (allowed-values k1om x86_64)
             (cardinality 2 2) (default (create$ x86_64 x86_64)))

  ; bandwidth in megabytes/secs
  (slot bandwidth (type FLOAT))

  ; local if node pair are on the same host, else remote
  (slot endpoints (type SYMBOL) (allowed-values local remote) (default remote))

  ; expected values for absolute threshold tests
  (slot expected-bandwidth (type FLOAT) (default 0.0))
  (slot expected-latency (type FLOAT) (default 1e6))

  ; latency in microseconds
  (slot latency (type FLOAT))
  (slot option_id (type STRING) (default "")))

(defmessage-handler IMB_PINGPONG statistics-key (?suffix)
  "Return a key that is unique for each combination of subcluster,
   role, architecture, option id, endpoint, and fabric."
  (bind ?role1 nil)
  (bind ?role2 nil)
  (bind ?subcluster1 nil)
  (bind ?subcluster2 nil)

  (bind ?ins1 (find-instance ((?n NODE))
                             (eq ?n:node_id (nth$ 1 ?self:node_id))))
  (if (= (length ?ins1) 1) then
    (bind ?i (nth$ 1 ?ins1))
    (bind ?subcluster1 (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role1 compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role1 enhanced))))

  (bind ?ins2 (find-instance ((?n NODE))
                             (eq ?n:node_id (nth$ 2 ?self:node_id))))
  (if (= (length ?ins2) 1) then
    (bind ?i (nth$ 1 ?ins2))
    (bind ?subcluster2 (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role2 compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role2 enhanced))))

  (switch ?self:i_mpi_fabrics
          (case dapl then
                (bind ?subfabric ?self:i_mpi_dapl_provider))
          (case ofa then
                (bind ?subfabric ?self:i_mpi_ofa_adapter_name))
          (case ofi then
                (bind ?subfabric ?self:i_mpi_ofi_provider))
          (case shm:dapl then
                (bind ?subfabric ?self:i_mpi_dapl_provider))
          (case shm:ofa then
                (bind ?subfabric ?self:i_mpi_ofa_adapter_name))
          (case shm:ofi then
                (bind ?subfabric ?self:i_mpi_ofi_provider))
          (case shm:tcp then
                (bind ?subfabric ?self:i_mpi_tcp_netmask))
          (case shm:tmi then
                (bind ?subfabric ?self:i_mpi_tmi_provider))
          (case tcp then
                (bind ?subfabric ?self:i_mpi_tcp_netmask))
          (case tmi then
                (bind ?subfabric ?self:i_mpi_tmi_provider))
          (default (bind ?subfabric nil)))

  (bind ?key (sym-cat (class ?self) _ ?subcluster1 _ ?subcluster2 _
                      ?role1 _ ?role2 _ ?self:option_id _ (nth$ 1 ?self:arch) _
                      (nth$ 2 ?self:arch) _ ?self:endpoints _
                      ?self:i_mpi_fabrics _ ?subfabric _ ?suffix))
  (return ?key))

(defmessage-handler IMB_PINGPONG init after ()
  (if (eq ?self:endpoints remote) then
    (send ?self add (send ?self statistics-key bandwidth) ?self:bandwidth)
    (send ?self add (send ?self statistics-key latency) ?self:latency)))
