 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass IOZONE
  (is-a BASE_NODE STATISTICS TIMESTAMP DATABASE)
  (role concrete)
  (pattern-match reactive)

  ; This value is read and updated by the analyze extension by parsing the
  ; STDOUT. This value is always positive, and 0.0 if the analyze extension's
  ; regex parsing didn't return anything
  (slot bandwidth (type NUMBER) (default 0))

  ; This value is set to true if the bandwidth is greater than 0.0 in
  ; the initialization message handler. This is then used to calculate
  ; statistics.
  (slot valid (type SYMBOL) (allowed-symbols TRUE FALSE UNKNOWN)
        (default UNKNOWN))

  ; This is the standard out as seen in the database.
  (slot stdout (type STRING)))

(defmessage-handler IOZONE init after ()
  "Add STATISTICS key based on the node architecture, e.g.,
   IOZONE-x86_64"
  (if (> ?self:bandwidth 0.0)
      then
    (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
    (send ?self add (send ?self statistics-key) ?self:bandwidth)
    (send ?self put-valid TRUE)))

(defmessage-handler IOZONE statistics-key ()
  (bind ?role nil)
  (bind ?subcluster nil)
  (bind ?arch nil)
  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then 
    (bind ?i (nth$ 1 ?ins))
    (bind ?arch (send ?i get-architecture))
    (bind ?subcluster (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) + ?subcluster + ?role + ?arch))
  (return ?key))

    

