 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass KERNEL_PARAM
  (is-a BASE_NODE BASE_TIMESTAMP DATABASE)
  (role concrete)
  (pattern-match reactive)

  ; the actual parameter
  (multislot params (type SYMBOL))
  ; the value for the parameter
  (multislot values (type STRING))
  ; set by analyze extension, used for the determining key insertion and running rules for a key
  ; more of a hint to optimize runtime
  (multislot status-list (type SYMBOL) (allowed-values uniform non-uniform extra missing)))

(defmessage-handler KERNEL_PARAM uniformity-key (?param)
  (bind ?role other)
  (if (member$ compute ?self:role) then (bind ?role compute))
  (if (member$ enhanced ?self:role) then (bind ?role enhanced))
  (bind ?key (sym-cat (class ?self) - ?self:subcluster - ?self:architecture +
                      ?role + ?param))
  (return ?key))

(defmessage-handler KERNEL_PARAM init after()
  ; Handle head nodes that are also compute nodes differently since
  ; they might have kernel parameters w.r.t compute nodes that are not present on
  ; dedicated compute nodes.
  (if (and (member$ head ?self:role)
           (or (member$ compute ?self:role) (member$ enhanced ?self:role)))
    then
    ; insert each kernel parameter individually, skipping the extra ones
    (progn$ (?param ?self:params)
            (bind ?state (send ?self get-status ?param))
            (if (neq ?state extra) then
                (multiset-insert (send ?self uniformity-key ?param) (send ?self get-value ?param))
            )
    )
    else
    ; for all other nodes, do a bulk insert
    (multiset-insert-multifield (send ?self uniformity-key "") ?self:params
                              ?self:values)))

(defmessage-handler KERNEL_PARAM get-value (?param)
  (bind ?i (member$ ?param ?self:params))
  (return (nth$ ?i ?self:values)))

(defmessage-handler KERNEL_PARAM get-status (?param)
  (bind ?i (member$ ?param ?self:params))
  (return (nth$ ?i ?self:status-list)))
