 ; Copyright 2018-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass LDCONFIG
  (is-a BASE_NODE BASE_TIMESTAMP DATABASE)
  (role concrete)
  (pattern-match reactive)

  (multislot libraries (type SYMBOL))
  (multislot architectures (type SYMBOL))
  (multislot versions (type SYMBOL))

  (multislot embree-libraries (storage shared) (type SYMBOL)
            (access read-only)
            (default (create$ libembree.so.2 libembree3.so.3)))

  (multislot embree-req-version (storage shared) (type SYMBOL)
            (default (create$ 2.17.4 3.2.0)))

  ; Required libdrm libraries and versions
  (multislot req-libdrm (storage shared) (type SYMBOL) (access read-only)
      (default (create$ libdrm.so.2 libdrm_amdgpu.so.1 libdrm_intel.so.1
                        libdrm_nouveau.so.2 libdrm_radeon.so.1 libkms.so.1)))

  (multislot libdrm-req-version (storage shared) (type SYMBOL)
            (default (create$ 2.4.0 1.0.0 1.0.0 2.0.0 1.0.1 1.0.0)))


  ; Required mesa libraries and versions
  (multislot req-mesa (storage shared) (type SYMBOL) (access read-only)
               (default (create$ libglapi.so.0 libGL.so.1 libOSMesa.so.8
                         libswrSKX.so.0 libswrAVX2.so.0 libswrAVX.so.0)))

  (multislot mesa-req-version (storage shared) (type SYMBOL)
            (default (create$ 0.0.0 1.5.0 8.0.0 0.0.0 0.0.0 0.0.0)))

  ; Required libfabric libraries
  (multislot req-libfabric (storage shared) (type SYMBOL) (access read-only)
      (default (create$ libfabric.so.1)))


  (multislot ospray-libraries (storage shared) (type SYMBOL)
             (default (create$ libospray.so.0 libospray_common.so.0
                      libospray_module_mpi.so.0
                      libospray_mpi_common.so.0
                      libospray_mpi_maml.so.0)))

  (multislot ospray-req-version (storage shared) (type SYMBOL)
             (default (create$ 1.6.0 1.6.0 1.6.0 1.6.0 1.6.0)))

  (multislot psxe-2018-libraries (storage shared) (type SYMBOL)
             (access read-only)
             (default (create$ libchkp.so libcilkrts.so.5 libgfxoffload.so
                      libicaf.so libifcoremt.so.5 libifcore.so.5 libifport.so.5
                      libimf.so libintlc.so.5 libioffload_host.so.5
                      libioffload_target.so.5 libiomp5.so libiompstubs5.so
                      libirc.so libirng.so libistrconv.so libmkl_avx2.so
                      libmkl_avx512_mic.so libmkl_avx512.so libmkl_avx.so
                      libmkl_blacs_intelmpi_ilp64.so
                      libmkl_blacs_intelmpi_lp64.so
                      libmkl_blacs_openmpi_ilp64.so
                      libmkl_blacs_openmpi_lp64.so
                      libmkl_blacs_sgimpt_ilp64.so libmkl_blacs_sgimpt_lp64.so
                      libmkl_cdft_core.so libmkl_core.so libmkl_def.so
                      libmkl_gf_ilp64.so libmkl_gf_lp64.so libmkl_gnu_thread.so
                      libmkl_intel_ilp64.so libmkl_intel_lp64.so
                      libmkl_intel_thread.so libmkl_mc3.so libmkl_mc.so
                      libmkl_pgi_thread.so libmkl_rt.so libmkl_scalapack_ilp64.so
                      libmkl_scalapack_lp64.so libmkl_sequential.so
                      libmkl_tbb_thread.so libmkl_vml_avx2.so
                      libmkl_vml_avx512_mic.so libmkl_vml_avx512.so
                      libmkl_vml_avx.so libmkl_vml_cmpt.so libmkl_vml_def.so
                      libmkl_vml_mc2.so libmkl_vml_mc3.so libmkl_vml_mc.so
                      libmpicxx.so.12 libmpifort.so.12 libmpi_gpfs.so.5.0
                      libmpi_ilp64.so.4 libmpijava.so.1 libmpi_lustre.so.5.0
                      libmpi_panfs.so.5.0 libmpi.so.12 libmpitune.so.1
                      libmpx.so liboffload.so.5 libpdbx.so.5 libprefork.so
                      libsvml.so libtbb_debug.so.2 libtbbmalloc_debug.so.2
                      libtbbmalloc_proxy_debug.so.2 libtbbmalloc_proxy.so.2
                      libtbbmalloc.so.2 libtbb_preview_debug.so.2
                      libtbb_preview.so.2 libtbb.so.2 libtmip_mx.so.1.0
                      libtmip_psm2.so.1.0 libtmip_psm.so.1.2 libtmi.so.1.2)))

  (multislot psxe-2019-libraries (storage shared) (type SYMBOL)
             (access read-only)
             (default (create$ libchkp.so libcilkrts.so.5
                      libicaf.so libifcoremt.so.5 libifcore.so.5 libifport.so.5
                      libimf.so libintlc.so.5 libioffload_host.so.5
                      libioffload_target.so.5 libiomp5.so libiompstubs5.so
                      libirc.so libirng.so libistrconv.so libmkl_avx2.so
                      libmkl_avx512_mic.so libmkl_avx512.so libmkl_avx.so
                      libmkl_blacs_intelmpi_ilp64.so
                      libmkl_blacs_intelmpi_lp64.so
                      libmkl_blacs_openmpi_ilp64.so
                      libmkl_blacs_openmpi_lp64.so
                      libmkl_blacs_sgimpt_ilp64.so libmkl_blacs_sgimpt_lp64.so
                      libmkl_cdft_core.so libmkl_core.so libmkl_def.so
                      libmkl_gf_ilp64.so libmkl_gf_lp64.so libmkl_gnu_thread.so
                      libmkl_intel_ilp64.so libmkl_intel_lp64.so
                      libmkl_intel_thread.so libmkl_mc3.so libmkl_mc.so
                      libmkl_pgi_thread.so libmkl_rt.so libmkl_scalapack_ilp64.so
                      libmkl_scalapack_lp64.so libmkl_sequential.so
                      libmkl_tbb_thread.so libmkl_vml_avx2.so
                      libmkl_vml_avx512_mic.so libmkl_vml_avx512.so
                      libmkl_vml_avx.so libmkl_vml_cmpt.so libmkl_vml_def.so
                      libmkl_vml_mc2.so libmkl_vml_mc3.so libmkl_vml_mc.so
                      libmpicxx.so.12 libmpifort.so.12
                      libmpi_ilp64.so.4.1 libmpijava.so.1
                      libmpi.so.12
                      libmpx.so liboffload.so.5 libpdbx.so.5
                      libsvml.so libtbb_debug.so.2 libtbbmalloc_debug.so.2
                      libtbbmalloc_proxy_debug.so.2 libtbbmalloc_proxy.so.2
                      libtbbmalloc.so.2 libtbb_preview_debug.so.2
                      libtbb_preview.so.2 libtbb.so.2))))

(defmessage-handler LDCONFIG find-arch (?lib)
  (bind ?i (member$ ?lib ?self:libraries))
  (return (nth$ ?i ?self:architectures)))

(defmessage-handler LDCONFIG find-version (?lib)
  (bind ?i (member$ ?lib ?self:libraries))
  (return (nth$ ?i ?self:versions)))

(defmessage-handler LDCONFIG find-embree-req-version (?lib)
  (bind ?i (member$ ?lib ?self:embree-libraries))
  (return (nth$ ?i ?self:embree-req-version)))

(defmessage-handler LDCONFIG find-ospray-req-version (?lib)
  (bind ?i (member$ ?lib ?self:ospray-libraries))
  (return (nth$ ?i ?self:ospray-req-version)))

(defmessage-handler LDCONFIG find-libdrm-req-version (?lib)
  (bind ?i (member$ ?lib ?self:req-libdrm))
  (return (nth$ ?i ?self:libdrm-req-version)))

(defmessage-handler LDCONFIG find-mesa-req-version (?lib)
  (bind ?i (member$ ?lib ?self:req-mesa))
  (return (nth$ ?i ?self:mesa-req-version)))
