 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass LSHW
  (is-a BASE_NODE BASE_TIMESTAMP DATABASE)
  (role concrete)
  (pattern-match reactive)

  (multislot builtin-blacklist (type STRING)
             (default (create$ "pci-usb.*" "scsi:*[0-9]*-driver"
                               "scsi:*[0-9]*-businfo" "scsi:*[0-9]*-logicalname"
                               "scsi:*[0-9]*-scsi-host")))
  (multislot keys (type STRING))
  (multislot values (type STRING))
  (multislot status-list (type SYMBOL) (allowed-values uniform non-uniform extra missing)))

(defmessage-handler LSHW init after()
  ; Handle head nodes that are also compute nodes differently since
  ; they might have extra keys w.r.t compute nodes that are not present on
  ; dedicated compute nodes.
  (if (and (member$ head ?self:role)
           (or (member$ compute ?self:role) (member$ enhanced ?self:role)))
    then
    ; insert each lshw key individually, skipping the extra ones
    (progn$ (?key ?self:keys)
            (bind ?state (send ?self get-status ?key))
            (if (neq ?state extra) then
                (multiset-insert (send ?self uniformity-key ?key) (send ?self get-value ?key))
            )
    )
    else
    ; for all other nodes, do a bulk insert
    (multiset-insert-multifield (send ?self uniformity-key "") ?self:keys
                              ?self:values)))

(defmessage-handler LSHW uniformity-key (?lkey)
  (bind ?role other)
  (if (member$ compute ?self:role) then (bind ?role compute))
  (if (member$ enhanced ?self:role) then (bind ?role enhanced))
  (bind ?key (sym-cat (class ?self) + ?self:subcluster + ?role + ?self:architecture + ?lkey))
  (return ?key))

(defmessage-handler LSHW get-value (?key)
  (bind ?i (member$ ?key ?self:keys))
  (return (nth$ ?i ?self:values)))

(defmessage-handler LSHW get-status (?key)
  (bind ?i (member$ ?key ?self:keys))
  (return (nth$ ?i ?self:status-list)))
