 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

;class for general lustre configuration/information on a node
(defclass LUSTRE
  (is-a BASE_NODE BASE_TIMESTAMP MULTISET DATABASE)
  (role concrete)
  (pattern-match reactive)

  ;indicates if the lustre kernel module is loaded
  (slot kernel_modules_loaded (type SYMBOL) (default no_data)
        (allowed-values ok error no_data)))


;class for metadata and object targets
(defclass LUSTRE_TARGET
  (is-a BASE_NODE BASE_TIMESTAMP MULTISET DATABASE)
  (role concrete)
  (pattern-match reactive)
  (slot target (type STRING))
  (slot active (type SYMBOL) (allowed-values TRUE FALSE) (default FALSE)))


;class for mount points that are accessible to the clients
(defclass LUSTRE_MOUNT_POINT
  (is-a BASE_NODE BASE_TIMESTAMP MULTISET DATABASE)
  (role concrete)
  (pattern-match reactive)

  ;name of the mount point
  (slot mount_point (type STRING))

  ;name of the filesystem
  (slot fs_name (type STRING))

  ;true if mount point is mounted on the node
  (slot mounted (type SYMBOL) (allowed-values TRUE FALSE) (default FALSE))

  ;stripe test total number of object targets
  (slot num_object_targets (type INTEGER) (default 0))

  ;stripe test number of object targets that have been written to
  (slot write_object_targets (type INTEGER) (default 0)))


(defmessage-handler LUSTRE_MOUNT_POINT init after ()
  (if (eq ?self:mounted TRUE) then
    (send ?self add (sym-cat (send ?self uniformity-key)
                             "-" ?self:mount_point "-write_targets")
          ?self:write_object_targets)))


(defmessage-handler LUSTRE_MOUNT_POINT uniformity-key ()
  (bind ?subcluster nil)
  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?subcluster (send ?i get-subcluster)))

  (bind ?key (sym-cat (class ?self) "-" ?subcluster "-" ?self:mount_point))
  (return ?key))

