 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass MOUNT
  (is-a BASE_NODE BASE_TIMESTAMP MULTISET DATABASE)
  (role concrete)
  (pattern-match reactive)

  (slot proc_mounted (type INTEGER) (default 0))
  (slot dev_shm_mounted (type INTEGER) (default 0))
  (slot home_path (type STRING) (default ""))
  (slot home_inode (type STRING) (default ""))
  (slot tmp_perms (type STRING) (default ""))
  (slot tmp_path (type STRING) (default ""))
  (slot pwd (type STRING) (default ""))
  (multislot filesystem (type SYMBOL))

  ; List of local file systems
  (multislot local-systems (storage shared) (type SYMBOL) (access read-only)
             (default (create$ ext2 ext3 ext4 xfs btrfs reiserfs tmpfs))))

(defmessage-handler MOUNT init after ()
  "Add MULTISET keys for home path and inode values"
  (if (> (str-length ?self:home_path) 0) then
    (send ?self add (sym-cat (send ?self uniformity-key) -home_path)
          ?self:home_path))

  (if (> (str-length ?self:home_inode) 0) then
    (send ?self add (sym-cat (send ?self uniformity-key) -home_inode)
          ?self:home_inode))

  (if (> (str-length ?self:tmp_path) 0) then
    (send ?self add (sym-cat (send ?self uniformity-key-basic) -tmp_path)
          ?self:tmp_path)))

(defmessage-handler MOUNT uniformity-key ()
  (bind ?role nil)
  (bind ?subcluster nil)
  (bind ?arch nil)
  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?arch (send ?i get-architecture))
    (bind ?subcluster (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) + ?subcluster + ?role + ?arch))
  (return ?key))

(defmessage-handler MOUNT uniformity-key-basic ()
  (bind ?subcluster nil)
  (bind ?arch nil)
  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?arch (send ?i get-architecture))
    (bind ?subcluster (send ?i get-subcluster)))

  (bind ?key (sym-cat (class ?self) + ?subcluster ?arch))
  (return ?key))
