 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass MPI_LOCAL
  "This class contains 2 sets of data - 1. The location of `mpirun` as
   output by `which mpirun` 2. The number of times the phrase `Hello
   world: rank [0-9] of [0-9]` occurs."
  (is-a BASE_NODE IMPI MULTISET TIMESTAMP DATABASE)
  (role concrete)
  (pattern-match reactive)

  ; The location of `mpirun` as output by `which mpirun` Note: If this
  ; fails, the assumption is that the Intel(R) Cluster Runtimes rules
  ; would have caught it already.
  (slot mpi_path (type STRING) (default ?NONE))

  ; This is populated by the analyze extension based on the number of times
  ; the phrase `Hello World: rank [0-9] of [0-9]` occurs. Note: 4 is
  ; hard coded, so we expect 4 lines to be here
  (slot valid_lines (type INTEGER) (default 0)))

(defmessage-handler MPI_LOCAL init after()
  "Add MULTISET keys for MPI_LOCAL. Skip cases where there was no mpi_path"
  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (> (str-length ?self:mpi_path) 0) then
    (send ?self add (send ?self uniformity-key) ?self:mpi_path)))

(defmessage-handler MPI_LOCAL uniformity-key ()
  (bind ?role nil)
  (bind ?subcluster nil)
  (bind ?arch nil)
  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?arch (send ?i get-architecture))
    (bind ?subcluster (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) + ?subcluster + ?role + ?arch))
  (return ?key))
