 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass OPA_CONFIG
  "Class for Intel(R) OPA software configuration settings."
  (is-a MULTISET TOOL-SET)
  (role concrete)
  (pattern-match reactive)
  (slot memlock (type INTEGER) (default 0)) ; The value -1 means "unlimited"
  (multislot req-tools (storage shared) (type SYMBOL) (access read-only)
             (default (create$ opasmaquery opahfirev saquery))))


(defmessage-handler OPA_CONFIG init after ()
  (send ?self add (send ?self multiset-key memlock) ?self:memlock))

(defmessage-handler OPA_CONFIG multiset-key (?v)
  ;; defaults
  (bind ?subcluster default)
  (bind ?role compute)
  (bind ?architecture x86_64)

  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?subcluster (send ?i get-subcluster))
    (bind ?architecture (send ?i get-architecture))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) + ?subcluster + ?role + ?architecture + ?v))
  (return ?key))

(defclass OPA_HFI_INFO
  "Class for Intel(R) OPA hardware infomation from opasmaquery, opahfirev"
  (is-a BASE_NODE BASE_TIMESTAMP DATABASE MULTISET)
  (role concrete)
  (pattern-match reactive)

  (slot base-lid (type INTEGER) (default 0))
  (slot ca (type SYMBOL) (default unknown))
  (slot capability-mask (type SYMBOL) (default unknown))
  (slot firmware-version (type SYMBOL) (default unknown))
  (slot hardware-version (type STRING) (default "unknown"))
  (slot lmc (type INTEGER) (default 0))
  (slot node-guid (type SYMBOL) (default unknown))
  (slot physical-state (type SYMBOL) (default unknown))
  (slot port (type INTEGER) (default 0))
  (slot port-guid (type SYMBOL) (default unknown))
  (slot rate (type NUMBER) (default 0))
  (slot sm-lid (type INTEGER) (default 0))
  (slot state (type SYMBOL) (default unknown))
  (slot system-image-guid (type SYMBOL) (default unknown))
  (slot error (type STRING) (default "")))

(defmessage-handler OPA_HFI_INFO init after ()
  (send ?self add (send ?self multiset-key ca) ?self:ca)
  (send ?self add (send ?self multiset-key firmware-version)
        ?self:firmware-version)
  (send ?self add (send ?self multiset-key hardware-version)
        ?self:hardware-version)
  (send ?self add (send ?self multiset-key physical-state) ?self:physical-state)
  (send ?self add (send ?self multiset-key rate) ?self:rate)
  (send ?self add (send ?self multiset-key state) ?self:state))

(defmessage-handler OPA_HFI_INFO multiset-key (?v)
  ;; defaults
  (bind ?subcluster default)
  (bind ?role compute)
  (bind ?architecture x86_64)

  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?subcluster (send ?i get-subcluster))
    (bind ?architecture (send ?i get-architecture))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) + ?self:ca + ?self:port + ?subcluster +
                      ?role + ?architecture + ?v))
  (return ?key))

(defclass OPA_PCI_DEVICE
  "Class for Intel(R) OPA PCI device data, i.e., lspci."
  (is-a BASE_NODE BASE_TIMESTAMP DATABASE MULTISET)
  (role concrete)
  (pattern-match reactive)

  (slot device (type STRING) (default "unknown"))
  (slot device_id (type SYMBOL) (default unknown))
  (slot driver (type SYMBOL) (default unknown))
  (slot physlot (type SYMBOL) (default unknown))
  (slot slot (type SYMBOL) (default unknown))
  (slot vendor (type STRING) (default "unknown"))
  (slot vendor_id (type SYMBOL) (default unknown)))

(defmessage-handler OPA_PCI_DEVICE init after ()
  (send ?self add (send ?self multiset-key device) ?self:device)
  (send ?self add (send ?self multiset-key driver) ?self:driver)
  (send ?self add (send ?self multiset-key physlot) ?self:physlot)
  (send ?self add (send ?self multiset-key vendor) ?self:vendor))

(defmessage-handler OPA_PCI_DEVICE multiset-key (?v)
  ;; defaults
  (bind ?subcluster default)
  (bind ?role compute)
  (bind ?architecture x86_64)

  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?subcluster (send ?i get-subcluster))
    (bind ?architecture (send ?i get-architecture))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) + ?self:slot + ?subcluster +
                      ?role + ?architecture + ?v))
  (return ?key))

(defclass OPA_FW_VER
  (is-a BASE_NODE BASE_TIMESTAMP DATABASE MULTISET)
  (role concrete)
  (pattern-match reactive)

  (multislot fw_ver (type STRING))
  (multislot hfi (type STRING)))

(defmessage-handler OPA_FW_VER init after ()
  (progn$ (?fw ?self:fw_ver)
    (send ?self add (send ?self uniformity-key fw_ver) ?fw)))

(defmessage-handler OPA_FW_VER uniformity-key (?v)
  (bind ?key (sym-cat (class ?self) + ?self:subcluster + ?self:architecture + ?v))
  (return ?key))
