 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass PERL
  "This class represents the data collected from a run of the perl
   benchmark.  The slots are populated by the analyze extension component
   based on the contents of the database."
  (is-a BASE_NODE MULTISET TIMESTAMP DATABASE)
  (role concrete)
  (pattern-match reactive)

  (slot version (type STRING) (default ?NONE))
  (slot stderr (type STRING) (default ?NONE)))

(defmessage-handler PERL init after ()
  "Add MULTISET keys for PERL.  Skip null version values."
  (if (> (str-length ?self:version) 0) then
    (send ?self add (send ?self uniformity-key) ?self:version)))

(defmessage-handler PERL uniformity-key ()
  (bind ?role nil)
  (bind ?subcluster nil)
  (bind ?arch nil)
  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?arch (send ?i get-architecture))
    (bind ?subcluster (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) + ?subcluster + ?role + ?arch))
  (return ?key))
