 ; Copyright 2018-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass PSXE_VERSIONS
  (is-a BASE_NODE BASE_TIMESTAMP DATABASE)
  (role concrete)
  (pattern-match reactive)

  ; This slot holds the name of the tool
  (multislot toolnames (type SYMBOL))

  ; This slot holds the tool's version if applicable
  (multislot versions (type STRING))

  ; Required tools
  (multislot req-tools (storage shared) (type SYMBOL) (access read-only)
      (default (create$ gcc gxx ifort mkl tbb mpi python2 python3)))

  ; Minimum versions for required tools
  (multislot req-versions (storage shared) (type STRING) (access read-only)
      (default (create$ "4.8.0" "3.4.18" "18.0" "2018.0" "2018" "2018"
                        "2018" "2018"))))

; Function to get a tool's version
(defmessage-handler PSXE_VERSIONS get-version (?tool)
  (bind ?i (member ?tool ?self:toolnames))
  (return (nth$ ?i ?self:versions)))

; Function to get a tool's required version
(defmessage-handler PSXE_VERSIONS get-req-version (?tool)
  (bind ?i (member ?tool ?self:req-tools))
  (return (nth$ ?i ?self:req-versions)))

; Function to get a tool's full string name
(defmessage-handler PSXE_VERSIONS get-tool-string (?tool)
  (if (eq (str-compare ?tool "gcc") 0) then
    (return "ANSI* standard C language runtime of the GNU* C Compiler"))
  (if (eq (str-compare ?tool "gxx") 0) then
    (return "ANSI* standard C++ language runtime of the GNU* C Compiler"))
  (if (eq (str-compare ?tool "ifort") 0) then
    (return "standard Fortran language runtime of the Intel(R) Fortran Compiler"))
  (if (eq (str-compare ?tool "mkl") 0) then
    (return "Intel(R) Math Kernel Library"))
  (if (eq (str-compare ?tool "tbb") 0) then
    (return "Intel(R) Threading Building Blocks"))
  (if (eq (str-compare ?tool "mpi") 0) then
    (return "Intel(R) MPI Library Runtime Environment"))
  (if (or (eq (str-compare ?tool "python2") 0)
          (eq (str-compare ?tool "python3") 0)) then
    (return "the Intel(R) Distribution for Python* scripting language"))
  (return ?tool))
