 ; Copyright 2017-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass SGEMM
  "This class represents the data collected from a run of the sgemm
   benchmark.  Since the 'S' in SGEMM is single precision, all the
   units are implicitly single precision.  Instances of the class are
   created, and the data slots are populated, by the analyze extension
   component."
  (is-a BASE_NODE BASE_TIMESTAMP STATISTICS DATABASE)
  (role concrete)
  (pattern-match reactive)

  (slot flops (type NUMBER) (default 0) ; units = double precision floating
        (range 0 ?VARIABLE))            ;         point ops per second
  (slot numactl (type STRING) (default ""))
  (slot peak (type NUMBER) (default 0)  ; units = same as flops slot
        (range 0 ?VARIABLE))
  (slot peak-fraction (type NUMBER) (default 0.9) (range 0 1))
  (slot taskset (type STRING) (default "")))

(defmessage-handler SGEMM init after ()
  (if (> ?self:flops 0) then
    (send ?self add (send ?self statistics-key) ?self:flops)))

(defmessage-handler SGEMM statistics-key ()
  "Return a key that is unique for each combination of subcluster,
   role, and architecture, e.g., SGEMM+sub+compute+x86_64."
  (bind ?role nil)
  (bind ?subcluster nil)

  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?subcluster (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) + ?subcluster + ?role + ?self:architecture))
  (return ?key))

(defmessage-handler SGEMM get-pk-fraction (?usrfraction ?calcfraction)
  "Determine which threshold fraction to use"
  (bind ?fraction ?calcfraction)
  (if (!= ?usrfraction 0) then
     (bind ?fraction ?usrfraction))
  (return ?fraction))
