 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass STREAM
  "This class represents the data collected from a run of the stream
   benchmark. Note, only the maximum value from a Triad run is saved."
  (is-a BASE_NODE STATISTICS TIMESTAMP DATABASE)
  (role concrete)
  (pattern-match reactive)

  ; This slot gets filled by the analyze extension based on the maximum Triad
  ; bandwidth value
  (slot triad (type NUMBER) (default 0))
  ; In this slot, we will look for the sequence "libiomp5.so not
  ; found".  If that is the case, we will tell the user Intel Runtimes
  ; need to be installed
  (slot stderr (type STRING) )
  ; In this slot, we will look for "Solution validates" and "Failed
  ; Validation" . If "Failed Validation" is found, we will tell the
  ; user to collect another set of data since something went wrong. If
  ; solution validates, we can use the current reading.
  (slot stdout (type STRING) )
  ; CLIPS populated this slot depending on whether "FAILED" or "VALID"
  ; is found in the stdout
  (slot valid (type SYMBOL) (allowed-symbols TRUE FALSE UNKNOWN)
        (default UNKNOWN)))

(defmessage-handler STREAM init after ()
  "Add STATISTICS key based on the node architecture, e.g.,
  STREAM-x86_64"
  (if (str-index "Solution Validates" ?self:stdout)
      then
    (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
    (send ?self add (send ?self statistics-key) ?self:triad)
    (send ?self put-valid TRUE)))

(defmessage-handler STREAM statistics-key ()
  (bind ?role nil)
  (bind ?subcluster nil)
  (bind ?arch nil)
  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?arch (send ?i get-architecture))
    (bind ?subcluster (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) + ?subcluster + ?role + ?arch))
  (return ?key))
