 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass TOOL-SET
   "This class represents any generic tool set. Not all slots need to be
   populated  since all slots have default values. The rules can
   decide which slots are necessary depending on the type of tool set

   Note: This slot is not derived from TIMESTAMP because there is no
   intention of doing time series analysis. Instead  the analyze extension
   will always populate the slots with the most recent data"
  (is-a BASE_NODE BASE_TIMESTAMP DATABASE)
  (role abstract)

  ;name of the toolset
  (slot set-name (type SYMBOL) (access read-only) (storage shared))

  ; This slot holds the name of the tool
  (multislot toolnames (type SYMBOL))

  ; This slot holds the tool's version  if applicable
  (multislot versions (type SYMBOL))

  ; This slot holds the stdout of a command run with the tool. It
  ; could be the output of running the command itself or it could be
  ; the output of running a script with the command etc
  (multislot stdouts (type STRING))

  ; This slot holds the stderr of a command run with the tool. It
  ; could be the standard error seen when running the command itself
  ; or it could be the standard error when running a script with the
  ; command etc.
  (multislot stderrs (type STRING)))

  (defmessage-handler TOOL-SET get-version (?tool)
    (bind ?i (member$ ?tool ?self:toolnames))
    (return (nth$ ?i ?self:versions)))

  (defmessage-handler TOOL-SET get-stdout (?tool)
    (bind ?i (member$ ?tool ?self:toolnames))
    (return (nth$ ?i ?self:stdouts)))

  (defmessage-handler TOOL-SET get-stderr (?tool)
    (bind ?i (member$ ?tool ?self:toolnames))
    (return (nth$ ?i ?self:stderrs)))

(defclass LSB-TOOL-SET
  (is-a TOOL-SET)
  (role concrete)
  (pattern-match reactive)
  ; slot defaulted to the name of this tool set
  (slot set-name (type SYMBOL) (source composite) (storage shared)
                               (default lsb_tools))
  ;ssf tools required within this tool set
  (multislot req-tools (storage shared) (type SYMBOL) (access read-only)
            (default (create$  [ ar awk basename bc cat
                                chmod chown cksum cmp comm cp
                                csplit cut date dd diff
                                dirname du echo egrep env expr
                                false fgrep file find fold
                                fuser getconf grep head hostname
                                iconv id join kill killall ln
                                logname ls mkdir mkfifo mktemp
                                more mv nice nl nohup od
                                paste patch pathchk pidof printf
                                ps pwd rm rmdir sed seq sh sleep
                                sort split strings tail tar tee
                                test time touch tr true uname
                                uniq wc xargs)))

  ;ssf tools required for 2018.0
  (multislot req-tools-2018 (storage shared) (type SYMBOL) (access read-only)
            (default (create$  [ ar awk basename bc cat
                                chmod chown cksum cmp comm cp
                                csplit cut date dd diff
                                dirname du echo ed egrep env ex expr
                                false fgrep file find fold
                                fuser getconf grep head hostname
                                iconv id join kill killall ln
                                logname ls mkdir mkfifo mktemp
                                more mv nice nl nohup od
                                paste patch pathchk pidof printf
                                ps pwd rm rmdir sed seq sh sleep
                                sort split strings tail tar tee
                                test time touch tr true uname
                                uniq vi wc xargs))))

(defclass SHELLS-TOOL-SET
  (is-a TOOL-SET)
  (role concrete)
  (pattern-match reactive)
  ; slot defaulted to the name of this tool set
  (slot set-name (type SYMBOL) (source composite) (storage shared)
                               (default shells)))
