 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass ANALYSIS-OUTPUT
  "Fundamental base class for out of specification conditions.  This
   class should be inherited by all classes that need the base
   characteristics to describe an out of specification condition, such
   as the confidence and severity of the condition."
  (is-a USER)

  (multislot node_id (type SYMBOL))
  (multislot args)
  (slot confidence (type NUMBER) (default 50.0) (range 0.0 100.0))
  (slot id (type STRING))
  (slot sub-id (type STRING))
  (slot remedy (type STRING))
  (multislot remedy-args)
  (slot severity (type NUMBER) (default 50.0) (range 0.0 100.0))
  (multislot source (type INSTANCE-ADDRESS)) ; instance(s) that are the
                                             ; basis of the ANALYSIS-OUTPUT
  (slot suppressed (type SYMBOL) (allowed-values TRUE FALSE) (default FALSE)))

(defmessage-handler ANALYSIS-OUTPUT put-source around ($?list)
  "Allow both instance names and instance addresses to be passed to
   put-source and guarantee that an address is ultimately stored."
  (bind ?new (create$ ))
  (progn$ (?i ?list)
          (bind ?new (insert$ ?new (+ (length$ ?new) 1) (instance-address ?i))))
  (override-next-handler ?new))
