 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass BASE_CLUSTER
  "Fundamental base class for clusters.  This class should be
   inherited by all classes that need the base characteristics of a
   cluster, such as multiple unique node identifiers."
  (is-a USER)
  (role abstract)
  (multislot node_id (type SYMBOL) (visibility public)))

(defclass CLUSTER
  "The core class for clusters.  This class describes the hardware and
   software that comprise a cluster and that is not node specific.
   For example, network switches."
  (is-a BASE_CLUSTER)
  (role concrete)
  (pattern-match reactive)
  (multislot nodes (type INSTANCE) (allowed-classes NODE)))

(defclass NODE_PAIR
  "This class describes the special cluster case of a pair of nodes
   that commonly reoccurs.  "
  (is-a CLUSTER)
  (role concrete)
  (pattern-match reactive)
  (multislot node_id (type SYMBOL) (cardinality 2 2) (visibility public))
  (multislot nodes (type INSTANCE) (allowed-classes NODE)
             (cardinality 2 2) (visibility public)))
