 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass CONFIG
  "Global configuration class.
   The CONFIG class allows users to parameterize the analysis engine
   in a single place.  There should only be one instance of this class
   created per clips environment."
  (is-a USER)
  (role concrete)
  (pattern-match reactive)

  ; The list of checks to be performed.  One to one mapping with
  ; analyze extensions of the same name.
  (multislot clck-checks (type SYMBOL) (default (create$)))

  (slot ssf-layer (type SYMBOL)
        (allowed-values core-2016.0 hpc-cluster-2016.0 compat-base-2016.0 compat-hpc-2016.0)
        (default core-2016.0))
  (slot cluster-mode-uniformity-threshold (type NUMBER) (default 0.9)
        (range 0 1))

  ; The maximum allowable age of a data point, in seconds, before a
  ; data point is considered "too old".  The default is 1 week.
  (slot data-age-threshold (type NUMBER) (default 604800))

  ;timestamp difference between dgemm data and cpu data for
  ;substandard-dgemm-due-to-offline-cores to activate
  (slot data-source-time-difference (type NUMBER) (default 900)
                                     (range 0 ?VARIABLE))

  ; The number of allowable median absolute deviations allowed before
  ; a dgemm value is considered an outlier.
  (slot dgemm-number-of-mad (type NUMBER) (default 6) (range 0 ?VARIABLE))

  ; The ratio of the measure dgemm performance to the theoretical peak
  ; value must be greater than or equal to this value.
  (slot dgemm-peak-fraction (type NUMBER) (default 0.0) (range 0 1))

  ; The performance threshold that a single node DGEMM has to meet
  (slot dgemm-perf-threshold  (type NUMBER) (default 2048))

  ; a sgemm value is considered an outlier.
  (slot sgemm-number-of-mad (type NUMBER) (default 6) (range 0 ?VARIABLE))

  ; The ratio of the measure sgemm performance to the theoretical peak
  ; value must be greater than or equal to this value.
  (slot sgemm-peak-fraction (type NUMBER) (default 0.0) (range 0 1))

  (multislot environment-blacklist (type STRING)
             (default (create$ "")))

  ; The performance threshold that a 4 node HPCG has to meet
  (slot hpcg-4node-perf-threshold  (type NUMBER) (default 126.9))

  ; The performance threshold that a 4 node HPL has to meet
  (slot hpl-4node-perf-threshold  (type NUMBER) (default 8050))

  ; The number of allowable median absolute deviations allowed before
  ; a hpl pairwise performance value is considered an outlier.
  (slot hpl-number-of-mad (type NUMBER) (default 6) (range 0 ?VARIABLE))

  (slot ssf-version (type SYMBOL NUMBER) (default 2016.0)
                    (allowed-values 2016.0))

  ; The performance threshold that the IMB pingpong bandwidth has to meet
  (slot imb_pingpong-bandwidth-perf-threshold  (type NUMBER) (default 11300))

  ; The performance threshold that the IMB pingpong latency has to meet
  (slot imb_pingpong-latency-perf-threshold  (type NUMBER) (default 1.9))

  (slot imb-pingpong-number-of-mad (type NUMBER) (default 6)
        (range 0 ?VARIABLE))

  ; The number of allowable median absolute deviations allowed before
  ; a iozone value is considered an outlier.
  (slot iozone-number-of-mad (type NUMBER) (default 6) (range 0 ?VARIABLE))

  (multislot kernel-blacklist (type STRING)
             (default (create$ "")))

  (multislot kernel-param-preferred (type STRING) (default (create$ "")))

  (slot kernel-param-uniformity-threshold (type NUMBER) (default 0.9)
        (range 0 1))

  (slot logical-cores-uniformity-threshold (type NUMBER) (default 0.9)
        (range 0 1))

  (multislot lshw-blacklist (type STRING)
             (default (create$ "")))

  (slot lshw-uniformity-threshold (type NUMBER) (default 0.9)
        (range 0 1))

  (slot memory-mode-uniformity-threshold (type NUMBER) (default 0.9)
        (range 0 1))

  ; The maximum allowable deviation in bytes from the median memory
  ; size before a memory size is considered non-uniform.
  (slot memory-uniformity-threshold (type NUMBER) (default 268435456)
        (range 0 ?VARIABLE))

  (slot ntp-offset-threshold (type NUMBER)
        (default 0.1))

  ; Maximum distance between the median and the median absolute deviation (MAD)
  ; for outlier detection measured in orders of magnitude.  If the difference is
  ; exceeded between the median and MAD then the MAD outlier algorithm iss
  ; disabled and a fallback mechanism is used for outlier checks.
  (slot outlier-max-median-mad-dist (type NUMBER) (default 2.5))

  ; Percentage of the median.  This is used to calculate a distance
  ; for the fallback outlier algorithm.  This distance is calculated by
  ; multiplying the median by the specified percentage.  If a sample is further
  ; away from the median than this distance, then it is considered an outlier.
  (slot outlier-median-pct (type NUMBER) (default 5) (range 0 ?VARIABLE))

  (slot preferred-cluster-mode (type SYMBOL)
        (allowed-values All2All SNC2 SNC4 Hemisphere Quadrant non-numa nil)
        (default nil))

  (slot preferred-memory-mode (type SYMBOL)
        (allowed-values Flat Cache Hybrid25 Hybrid50 numa nil)
        (default nil))

  (slot preferred-tickless-cores (type STRING)
        (default ""))

  (slot preferred-turbo-status (type SYMBOL)
        (allowed-values enabled disabled nil)
        (default nil))

  (multislot preferred-services-status (type STRING)
             (default (create$ "")))

  (slot rpm-uniformity-threshold (type NUMBER) (default 0.9)
        (range 0 1))

  ; The performance threshold that a single node HPCG has to meet
  (slot hpcg-single-perf-threshold  (type NUMBER) (default 32.0))

  ; The maximum used disk space
  (slot storage-max-used-pct (type INTEGER) (default 85) (range 0 ?VARIABLE))

  ; The number of allowable median absolute deviations allowed before
  ; a stream value is considered an outlier.
  (slot stream-number-of-mad (type NUMBER) (default 6) (range 0 ?VARIABLE))

  ; The performance threshold that STREAM Triad has to meet
  (slot stream-perf-threshold  (type NUMBER) (default 164000))

  (slot threads-per-core-uniformity-threshold (type NUMBER) (default 0.9)
        (range 0 1))

  (slot turbo-status-uniformity-threshold (type NUMBER) (default 0.9)
        (range 0 1))
)


(defrule default-config
  "Create the [config] instance if one does not already exist."
  (not (object (is-a CONFIG) (name [config])))
  =>
  (make-instance [config] of CONFIG))
