 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass IMPI
  "Base class to contain Intel(R) MPI Library runtime configuration
   settings.  Slot names directly correspond to Intel(R) MPI Library
   environment variable names, but in lower case.  This class should
   be inherited by all classes that have associated Intel(R) MPI
   Library settings."
  (is-a USER)
  (role abstract)

  (slot i_mpi_dapl_provider (type SYMBOL) (default ofa-v2-mlx4_0-1)
        (visibility public))
  (slot i_mpi_fabrics (type SYMBOL) (default shm:dapl)
        (allowed-values dapl ofa shm shm:dapl shm:ofa shm:tcp shm:tmi shm:ofi tcp tmi ofi)
        (visibility public))
  (slot i_mpi_ofa_adapter_name (type SYMBOL) (default mlx4)
        (visibility public))
  (slot i_mpi_ofi_provider (type SYMBOL) (default auto)
        (visibility public))
  (slot i_mpi_tcp_netmask (type SYMBOL) (default eth0)
        (visibility public))
  (slot i_mpi_tmi_provider (type SYMBOL) (default psm)
        (allowed-values mx psm psm2)
        (visibility public)))

(defmessage-handler IMPI get-impi-setting-string ()
  "Return a string containing the most relevant Intel(R) MPI Library
   settings."
  (bind ?string (str-cat "I_MPI_FABRICS=" ?self:i_mpi_fabrics))
  (return ?string))
