 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass NETWORK_ADAPTER
  "Base class for physical network adapters."
  (is-a USER)
  (role concrete)

  (slot mpi-dapl-bandwidth (type FLOAT) (default 1.0)) ; units = MB/s
  (slot mpi-dapl-latency (type FLOAT) (default 1e6))   ; units = microseconds
  (slot mpi-ofa-bandwidth (type FLOAT) (default 1.0))  ; units = MB/s
  (slot mpi-ofa-latency (type FLOAT) (default 1e6))    ; units = microseconds
  (slot mpi-ofi-bandwidth (type FLOAT) (default 1.0))  ; units = MB/s
  (slot mpi-ofi-latency (type FLOAT) (default 1e6))   ; units = microseconds
  (slot mpi-tcp-bandwidth (type FLOAT) (default 1.0))  ; units = MB/s
  (slot mpi-tcp-latency (type FLOAT) (default 1e6))    ; units = microseconds
  (slot mpi-tmi-bandwidth (type FLOAT) (default 1.0))  ; units = MB/s
  (slot mpi-tmi-latency (type FLOAT) (default 1e6))    ; units = microseconds
  (slot pci_id (type SYMBOL))
  (slot pci_name (type STRING)))

(defclass ETHERNET_ADAPTER
  "Base class for physical Ethernet adapters."
  (is-a NETWORK_ADAPTER)
  (role concrete)

  (slot speed (type SYMBOL) (allowed-values 1GbE 10GbE 40GbE 100GbE)
        (default 1GbE) (visibility public)))

(defclass INFINIBAND_ADAPTER
  "Base class for physical InfiniBand adapters."
  (is-a NETWORK_ADAPTER)
  (role concrete)

  (slot speed (type SYMBOL) (allowed-values SDR DDR QDR FDR EDR)
        (default QDR) (visibility public)))

(defclass INTEL-ETHERNET_ADAPTER
  "Class for Intel Ethernet adapters."
  (is-a ETHERNET_ADAPTER)
  (role concrete)
  (pattern-match reactive))

(defclass INTEL-INFINIBAND_ADAPTER
  "Class for Intel InfiniBand adapters."
  (is-a INFINIBAND_ADAPTER)
  (role concrete)
  (pattern-match reactive))

(defclass MELLANOX-INFINIBAND_ADAPTER
  "Class for Mellanox InfiniBand adapters."
  (is-a INFINIBAND_ADAPTER)
  (role concrete)
  (pattern-match reactive))

(defclass INTEL-OPA_ADAPTER
  "Class for Intel OPA InfiniBand adapters."
  (is-a INFINIBAND_ADAPTER)
  (role concrete)
  (pattern-match reactive))
