 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass SIGN
  "Abstract class to describe signs discovered by examining
   observations.  Derived from the ANALYSIS-OUTPUT class to get confidence,
   severity, etc. slots.  The state slot transitions from initialized,
   when the sign is created, to observed when it is first raised by a
   rule, and finally to diagnosed when it is used to make a higher
   level diagnosis."
  (is-a ANALYSIS-OUTPUT)
  (role abstract)
  (pattern-match non-reactive)

  (slot state (type SYMBOL) (allowed-values initialized observed diagnosed)
        (default initialized) (visibility public))
  (slot value))

(defclass BOOLEAN_SIGN
  "Specialized sign class to contain boolean signs."
  (is-a SIGN)
  (role concrete)
  (pattern-match reactive)

  (slot value (type SYMBOL) (allowed-values TRUE FALSE) (default FALSE)
        (source exclusive)))

(defclass COUNTER_SIGN
  "Specialized sign class to contain counter signs."
  (is-a SIGN)
  (role concrete)
  (pattern-match reactive)

  (slot value (type INTEGER) (default 0) (source exclusive)))

(defclass GENERIC_SIGN
  "General sign class to contain arbitrary signs.  Use one of the
   specialized sign classes if possible."
  (is-a SIGN)
  (role concrete)
  (pattern-match reactive))

(defclass PERFORMANCE_SIGN
  "Specialized sign class to contain performance signs."
  (is-a SIGN)
  (role concrete)
  (pattern-match reactive)

  (slot value (type SYMBOL) (allowed-values normal substandard invalid)
        (default normal) (source exclusive)))

(defmessage-handler COUNTER_SIGN increment ()
  "Increment the COUNTER_SIGN value slot by 1."
  (bind ?self:value (+ 1 ?self:value)))

(defclass DEPENDENT_SIGN
  "Specialized sign class to create dependent signs."
  (is-a SIGN)
  (role concrete)
  (pattern-match reactive)
  ; default error code is one
  (slot error_code (type INTEGER) (default 1))
  ; option id field is to take care of option set execution
  (slot OptionID (type SYMBOL)))

