 ; Copyright 2006-2019 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(deffunction vercmp (?v1 ?v2)
 "Compare two versions, v1 and v2 and return -1 if v1 < v2, 0 if v1
 == v2, or 1 if v1 > v2.  This particular implementation assumes
 that all versions are composed purely of numbers (i.e., no letters,
     dashes, etc.) separated by periods."
 (bind ?t1 (tokenize "." ?v1))
 (bind ?t2 (tokenize "." ?v2))

 (bind ?len (max (length$ ?t1) (length$ ?t2)))

 (loop-for-count (?i 1 ?len) do
  (bind ?e1 (version-field ?i ?t1))
  (bind ?e2 (version-field ?i ?t2))
  (if (> ?e1 ?e2) then (return 1))
  (if (> ?e2 ?e1) then (return -1)))

 (return 0))
