; Copyright 2006-2019 Intel Corporation
;
; This software and the related documents are Intel copyrighted materials, and
; your use of them is governed by the express license under which they were
; provided to you (License). Unless the License provides otherwise, you may
; not use, modify, copy, publish, distribute, disclose, or transmit this
; software or the related documents without Intel's prior written permission.
;
; This software and the related documents are provided as is, with no express
; or implied warranties, other than those that are expressly stated in the
; License.

(defrule not-intel-ssf-compliant-initial-2016.0
  "If there are any signs for Intel(R) Scalable System Framework 2016.0
   non-compliance, create a not-INTEL-SSF diagnosis and mark the sign as diagnosed.
   This rule only fires for the first non-compliance sign per node, i.e.,
   when the diagnosis does not already exist. Once the diagnosis
   exists, it should not be duplicated. Thus, there is a
   corresponding rule, not-intel-ssf-compliant-subsequent-2016.0, for the case
   where there are multiple signs leading to this diagnosis."
   (object (is-a CONFIG) (name [config]) (ssf-version 2016.0))
   ?sign <- (object (is-a SIGN) (node_id ?node)
                   (id "compat-base-2016-missing" |
                       "compat-hpc-2016-missing" |
                       "core-2016-missing" |
                       "cpu-not-intel64" |
                       "kernel-not-ssf-core-2016.0" |
                       "memory-minimum-required-compat-base" |
                       "memory-minimum-required-compat-hpc" |
                       "missing-lsb-common-tool" |
                       "missing-shell-ssf" |
                       "missing-ssf-library" |
                       "mount-home-not-defined" |
                       "mount-tmpdir-not-defined" |
                       "mpi-local-path-not-uniform" |
                       "perl-not-found" |
                       "perl-not-ssf-2016.0" |
                       "processor-not-intel64" |
                       "python-not-found" |
                       "python-not-ssf-2016.0" |
                       "ssf-file-not-found" |
                       "ssf-file-other-error" |
                       "ssf-layer-compat-base-2016.0" |
                       "ssf-layer-compat-hpc-2016.0" |
                       "ssf-layer-core-2016.0" |
                       "ssf-layer-hpc-cluster-2016.0" |
                       "storage-compute" |
                       "storage-head" |
                       "tcl-not-found" |
                       "tcl-not-ssf-2016.0")
                    (state ~diagnosed) (suppressed FALSE))
  (not (object (is-a INTEL_SSF_DIAGNOSIS) (node_id ?node) (id "not-intel-ssf-compliant-2016.0")))
   =>
   (send ?sign put-state diagnosed)
   (make-instance of INTEL_SSF_DIAGNOSIS (node_id ?node)
                  (confidence (send ?sign get-confidence))
                  (severity (send ?sign get-severity))
                  (id "not-intel-ssf-compliant-2016.0")
                  (source (send ?sign get-source))))


(defrule not-intel-ssf-compliant-subsequent-2016.0
  "This rule is related to not-intel-ssf-compliant-initial-2016.0. The difference
   is that this rule fires only after the initial diagnosis has
   already been created.  This rule marks the sign as diagnosed, and
   also adds to the list of signs that produced the diagnosis."
  (object (is-a CONFIG) (name [config]) (ssf-version 2016.0))
  ?diag <- (object (is-a INTEL_SSF_DIAGNOSIS) (node_id ?node) (id "not-intel-ssf-compliant-2016.0"))
  ?sign <- (object (is-a SIGN) (node_id ?node)
                   (id "compat-base-2016-missing" |
                       "compat-hpc-2016-missing" |
                       "core-2016-missing" |
                       "cpu-not-intel64" |
                       "kernel-not-ssf-core-2016.0" |
                       "memory-minimum-required-compat-base" |
                       "memory-minimum-required-compat-hpc" |
                       "missing-lsb-common-tool" |
                       "missing-ssf-library" |
                       "mount-home-not-defined" |
                       "mount-tmpdir-not-defined" |
                       "mpi-local-path-not-uniform" |
                       "perl-not-found" |
                       "perl-not-ssf-2016.0" |
                       "processor-not-intel64" |
                       "python-not-found" |
                       "python-not-ssf-2016.0" |
                       "ssf-file-not-found" |
                       "ssf-file-other-error" |
                       "ssf-layer-compat-base-2016.0" |
                       "ssf-layer-compat-hpc-2016.0" |
                       "ssf-layer-core-2016.0" |
                       "ssf-layer-hpc-cluster-2016.0" |
                       "storage-compute" |
                       "storage-head" |
                       "tcl-not-found" |
                       "tcl-not-ssf-2016.0")
                    (state ~diagnosed) (suppressed FALSE))
  =>
  (send ?sign put-state diagnosed)
  (send ?diag put-confidence (combine-confidence (send ?diag get-confidence)
                                                 (send ?sign get-confidence)))
  (send ?diag put-severity (combine-severity (send ?diag get-severity)
                                             (send ?sign get-severity)))
  (slot-insert$ ?diag source 1 (send ?sign get-source)))

