; Copyright 2006-2019 Intel Corporation
;
; This software and the related documents are Intel copyrighted materials, and
; your use of them is governed by the express license under which they were
; provided to you (License). Unless the License provides otherwise, you may
; not use, modify, copy, publish, distribute, disclose, or transmit this
; software or the related documents without Intel's prior written permission.
;
; This software and the related documents are provided as is, with no express
; or implied warranties, other than those that are expressly stated in the
; License.

(defrule iozone-outlier
  "Locate values that are outliers.  An outlier is a value that is
   outside the range defined by the median +/- 6 * median absolute
   deviation.  The statistics are computed using all samples on all
   nodes (i.e., use the IOZONE statistics key).

   Note: the statistics_control condition is required to ensure that
   all samples are included when computing the statistics."
  (object (is-a CONFIG) (name [config])
          (clck-checks $? iozone $?) (iozone-number-of-mad ?num-mad)
          (outlier-max-median-mad-dist ?med-mad-dist)
          (outlier-median-pct ?median-pct))

  ?obj <- (object (is-a IOZONE) (node_id ?node) (bandwidth ?bw)
                  (statistics_control TRUE) (latest-timestamp TRUE)
                  (valid TRUE))
  (object (is-a NODE) (node_id ?node))

  (test (send ?obj outlier (send ?obj statistics-key) ?bw ?num-mad
              ?med-mad-dist ?median-pct))

  =>

  (bind ?key (send ?obj statistics-key))
  (bind ?stat-instance (send ?obj _find_pair ?key))

  (bind ?median (send ?stat-instance median))
  (bind ?margin (send ?stat-instance get-outlier-margin))

  (bind ?severity (send ?obj outlier-severity ?key ?bw 25 60))

  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 75) (severity ?severity) (state observed)
                 (id "iozone-outlier") (source ?obj)
                 (args (create$ ?bw ?median ?margin ))))
