; Copyright 2006-2019 Intel Corporation
;
; This software and the related documents are Intel copyrighted materials, and
; your use of them is governed by the express license under which they were
; provided to you (License). Unless the License provides otherwise, you may
; not use, modify, copy, publish, distribute, disclose, or transmit this
; software or the related documents without Intel's prior written permission.
;
; This software and the related documents are provided as is, with no express
; or implied warranties, other than those that are expressly stated in the
; License.

(defrule process-is-a-zombie
  "For the most recent PROCESS data point, identify nodes with zombie
   processes, i.e., processes with a 'Z' state."
  (object (is-a CONFIG) (name [config])
          (clck-checks $? process $?))
  ?obj <- (object (is-a PROCESS) (node_id ?node) (etime ?etime)
                  (latest-timestamp TRUE) (pid ?pid) (state ?state) (uid ?uid)
                  (username ?uname) (cmd ?command))
  (object (is-a NODE) (node_id ?node))
  (test (= (str-compare ?state "Z") 0))
  =>
  ; the longer the zombie has been running, the higher the severity
  (bind ?severity (* (min 1 (/ ?etime 3600)) 100))
  (make-instance of BOOLEAN_SIGN
                 (node_id ?node) (confidence 90) (severity ?severity)
                 (state observed) (value TRUE) (source ?obj)
                 (id "process-is-a-zombie") (sub-id (str-cat ?command))
                 (args (create$ ?command ?uname ?etime ?pid))
                 (remedy "process-is-zombie-remedy")))

