; Copyright 2017-2019 Intel Corporation
;
; This software and the related documents are Intel copyrighted materials, and
; your use of them is governed by the express license under which they were
; provided to you (License). Unless the License provides otherwise, you may
; not use, modify, copy, publish, distribute, disclose, or transmit this
; software or the related documents without Intel's prior written permission.
;
; This software and the related documents are provided as is, with no express
; or implied warranties, other than those that are expressly stated in the
; License.

(defrule sgemm-data-is-substandard
  "For the most recent SGEMM data point, identify nodes with
   substandard FLOPS relative to a threshold based on the hardware.
   The severity depends on the amount of deviation from the threshold
   value; the larger the deviation, the higher the severity."
  (object (is-a CONFIG) (name [config])
          (clck-checks $? sgemm $?)
          (sgemm-peak-fraction ?fraction))

  ?obj <- (object (is-a SGEMM) (flops ?flops&~0) (node_id ?node) (peak ?peak&~0)
                  (peak-fraction ?pfraction))

  (object (is-a NODE) (node_id ?node)
          (role $?role&:(member$ compute ?role)|:(member$ enhanced ?role)))

  (test (< (/ ?flops ?peak) (send ?obj get-pk-fraction ?fraction ?pfraction)))
  =>
  (bind ?severity (* (- 1 (gaussian ?flops (* (send ?obj get-pk-fraction ?fraction ?pfraction) ?peak)
                                    (/ ?peak 10))) 80))
  (make-instance of PERFORMANCE_SIGN (node_id ?node) (confidence 90)
                 (severity ?severity) (state observed) (value substandard)
                 (id "sgemm-data-is-substandard")
                 (source ?obj)
                 (args (create$ ; convert FLOPS to GFLOPS
                        (/ ?flops 1e9) (* 100 (/ ?flops ?peak))
                        (/ ?peak 1e9)))))
