; Copyright 2017-2019 Intel Corporation
;
; This software and the related documents are Intel copyrighted materials, and
; your use of them is governed by the express license under which they were
; provided to you (License). Unless the License provides otherwise, you may
; not use, modify, copy, publish, distribute, disclose, or transmit this
; software or the related documents without Intel's prior written permission.
;
; This software and the related documents are provided as is, with no express
; or implied warranties, other than those that are expressly stated in the
; License.

(defrule sgemm-outlier
  "Locate values that are outliers.  An outlier is a value that is
   outside the range defined by the median +/- 6 * median absolute
   deviation.  The statistics are computed using all samples on all
   nodes (i.e., use the SGEMM statistics key).

   Note: the statistics_control condition is required to ensure that
   all samples are included when computing the statistics."
  (object (is-a CONFIG) (name [config])
          (clck-checks $? sgemm $?) (sgemm-number-of-mad ?num-mad)
          (outlier-max-median-mad-dist ?med-mad-dist)
          (outlier-median-pct ?median-pct))
  ?obj <- (object (is-a SGEMM) (node_id ?node) (flops ?flops&~0)
                  (statistics_control TRUE))
  (object (is-a NODE) (node_id ?node)
          (role $?role&:(member$ compute ?role)|:(member$ enhanced ?role))
          (subcluster ?subcluster))
  (test (send ?obj outlier (send ?obj statistics-key) ?flops ?num-mad
              ?med-mad-dist ?median-pct))
  =>

  (bind ?key (send ?obj statistics-key))
  (bind ?stat-instance (send ?obj _find_pair ?key))

  (bind ?median (send ?stat-instance median))
  (bind ?margin (send ?stat-instance get-outlier-margin))

  (bind ?severity (send ?obj outlier-severity ?key ?flops 25 60))

  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 75) (severity ?severity) (state observed)
                 (id "sgemm-outlier") (source ?obj)
                 (args (create$ (/ ?flops 1e9) (/ ?median 1e9)
                                (/ ?margin 1e9)) ?subcluster)))

