; Copyright 2017-2019 Intel Corporation
;
; This software and the related documents are Intel copyrighted materials, and
; your use of them is governed by the express license under which they were
; provided to you (License). Unless the License provides otherwise, you may
; not use, modify, copy, publish, distribute, disclose, or transmit this
; software or the related documents without Intel's prior written permission.
;
; This software and the related documents are provided as is, with no express
; or implied warranties, other than those that are expressly stated in the
; License.

(defrule data-is-too-old-initial
  "If there are any signs for out of date data, create a data is too
   old diagnosis and mark the sign as diagnosed.  This rule only fires
   for the first data is too old sign per node, i.e., when the
   diagnosis does not already exist.  Once the diagnosis exists, it
   should not be duplicated.  Thus, there is a corresponding rule,
   data-is-too-old-subsequent, for the case where there are multiple
   signs leading to this diagnosis."
  (object (is-a CONFIG) (name [config]))
  ?sign <- (object (is-a SIGN) (node_id $?node)
                               (id "all-to-all-data-is-too-old" |
                               "cpu-data-is-too-old" |
                               "datconf-data-is-too-old" |
                               "dgemm-data-is-too-old" |
                               "environment-data-is-too-old" |
                               "ethernet-data-is-too-old" |
                               "hpl-data-is-too-old" |
                               "imb-pingpong-data-is-too-old" |
                               "infiniband-data-is-too-old" |
                               "iozone-data-is-too-old" |
                               "kernel-data-is-too-old" |
                               "kernel-param-data-is-too-old" |
                               "ldconfig-data-is-too-old" |
                               "lsb_tools-data-is-too-old" |
                               "lshw-data-is-too-old" |
                               "lustre-data-is-too-old" |
                               "memory-data-is-too-old" |
                               "mount-data-is-too-old" |
                               "mpi-internode-data-is-too-old" |
                               "mpi-local-data-is-too-old" |
                               "ntp-data-is-too-old" |
                               "opa-data-is-too-old" |
                               "perl-data-is-too-old" |
                               "process-data-is-too-old" |
                               "psxe_versions-data-is-too-old" |
                               "python-data-is-too-old" |
                               "rpm-data-is-too-old" |
                               "services-data-is-too-old" |
                               "sgemm-data-is-too-old" |
                               "shells-data-is-too-old" |
                               "libraries-data-is-too-old" |
                               "ssf_version-data-is-too-old" |
                               "storage-data-is-too-old" |
                               "stream-data-is-too-old" |
                               "tcl-data-is-too-old" |
                               "xp-modes-data-is-too-old")
                   (state ~diagnosed) (suppressed FALSE))
  (not (object (is-a SHARED_DIAGNOSIS) (node_id $?node) (id "data-is-too-old")))
  =>
  (send ?sign put-state diagnosed)
  (make-instance of SHARED_DIAGNOSIS (node_id $?node)
                 (confidence (send ?sign get-confidence))
                 (severity (send ?sign get-severity))
                 (id "data-is-too-old")
                 (source (send ?sign get-source))
                 (remedy "data-is-too-old-remedy")))

(defrule data-is-too-old-subsequent
  "This rule is related to data-is-too-old-initial.  The difference is
   that this rule fires only after the initial diagnosis has already
   been created.  This rule marks the sign as diagnosed, and also adds
   to the list of signs that produced the diagnosis."
  (object (is-a CONFIG) (name [config]))
  ?diag <- (object (is-a SHARED_DIAGNOSIS) (node_id $?node) (id "data-is-too-old"))
  ?sign <- (object (is-a SIGN) (node_id $?node)
                               (id "all-to-all-data-is-too-old" |
                               "cpu-data-is-too-old" |
                               "datconf-data-is-too-old" |
                               "dgemm-data-is-too-old" |
                               "environment-data-is-too-old" |
                               "ethernet-data-is-too-old" |
                               "hpl-data-is-too-old" |
                               "imb-pingpong-data-is-too-old" |
                               "infiniband-data-is-too-old" |
                               "iozone-data-is-too-old" |
                               "kernel-data-is-too-old" |
                               "kernel-param-data-is-too-old" |
                               "ldconfig-data-is-too-old" |
                               "lsb_tools-data-is-too-old" |
                               "lshw-data-is-too-old" |
                               "lustre-data-is-too-old" |
                               "memory-data-is-too-old" |
                               "mount-data-is-too-old" |
                               "mpi-internode-data-is-too-old" |
                               "mpi-local-data-is-too-old" |
                               "ntp-data-is-too-old" |
                               "opa-data-is-too-old" |
                               "perl-data-is-too-old" |
                               "process-data-is-too-old" |
                               "psxe_versions-data-is-too-old" |
                               "python-data-is-too-old" |
                               "rpm-data-is-too-old" |
                               "services-data-is-too-old" |
                               "sgemm-data-is-too-old" |
                               "shells-data-is-too-old" |
                               "libraries-data-is-too-old" |
                               "ssf_version-data-is-too-old" |
                               "storage-data-is-too-old" |
                               "stream-data-is-too-old" |
                               "tcl-data-is-too-old" |
                               "xp-modes-data-is-too-old")
                   (state ~diagnosed) (suppressed FALSE))
  =>
  (send ?sign put-state diagnosed)
  (send ?diag put-confidence (combine-confidence (send ?diag get-confidence)
                                                 (send ?sign get-confidence)))
  (send ?diag put-severity (combine-severity (send ?diag get-severity)
                                             (send ?sign get-severity)))
  (slot-insert$ ?diag source 1 (send ?sign get-source)))

(defrule no-data-initial
  "If there are any signs for missing data, create a no data diagnosis
   and mark the sign as diagnosed.  This rule only fires for the first
   no data sign per node, i.e., when the diagnosis does not already
   exist.  Once the diagnosis exists, it should not be duplicated.
   Thus, there is a corresponding rule, no-data-subsequent, for the
   case where there are multiple signs leading to this diagnosis."
  (object (is-a CONFIG) (name [config]))
  ?sign <- (object (is-a SIGN) (node_id $?node)
                   (id "all_to_all-data-missing" |
                   "cpu-data-missing" |
                   "datconf-data-missing" |
                   "dgemm-data-missing" |
                   "dmidecode-data-missing" |
                   "environment-data-missing" |
                   "ethernet-data-missing" |
                   "hpcg-cluster-data-missing" |
                   "hpcg-single-data-missing" |
                   "hpl-data-missing" |
                   "imb_pingpong-data-missing" |
                   "infiniband-data-missing" |
                   "intel-pstate-data-missing" |
                   "iozone-data-missing" |
                   "kernel-data-missing" |
                   "kernel_param-data-missing" |
                   "ldconfig-data-missing" |
                   "libraries-data-missing" |
                   "lsb_tools-data-missing" |
                   "lscpu-data-missing" |
                   "lspci_verbose_data_missing" |
                   "lshw-data-missing" |
                   "lustre-data-missing" |
                   "memory-data-missing" |
                   "mount-data-missing" |
                   "mpi_internode-data-missing" |
                   "mpi_local-data-missing" |
                   "ntp-data-missing" |
                   "opa-data-missing" |
                   "perl-data-missing" |
                   "process-data-missing" |
                   "python-data-missing" |
                   "rpm-data-missing" |
                   "services-data-missing" |
                   "sgemm-data-missing" |
                   "shells-data-missing" |
                   "ssf_version-data-missing" |
                   "storage-data-missing" |
                   "stream-data-missing" |
                   "tcl-data-missing" |
                   "xp-modes-data-missing")
                   (state ~diagnosed) (suppressed FALSE))
  (not (object (is-a SHARED_DIAGNOSIS) (node_id $?node) (id "no-data")))
  =>
  (send ?sign put-state diagnosed)
  (make-instance of SHARED_DIAGNOSIS (node_id ?node)
                 (confidence (send ?sign get-confidence))
                 (severity (send ?sign get-severity))
                 (id "no-data") (source (send ?sign get-source))
                 (remedy "no-data-remedy")))

(defrule no-data-subsequent
  "This rule is related to no-data-initial.  The difference is that
   this rule fires only after the initial diagnosis has already been
   created.  This rule marks the sign as diagnosed, and also adds to
   the list of signs that produced the diagnosis."
  (object (is-a CONFIG) (name [config]))
  ?diag <- (object (is-a SHARED_DIAGNOSIS) (node_id $?node) (id "no-data"))
  ?sign <- (object (is-a SIGN) (node_id $?node)
                     (id "all_to_all-data-missing" |
                     "cpu-data-missing" |
                     "datconf-data-missing" |
                     "dgemm-data-missing" |
                     "dmidecode-data-missing" |
                     "environment-data-missing" |
                     "ethernet-data-missing" |
                     "hpcg-cluster-data-missing" |
                     "hpcg-single-data-missing" |
                     "hpl-data-missing" |
                     "imb_pingpong-data-missing" |
                     "infiniband-data-missing" |
                     "intel-pstate-data-missing" |
                     "iozone-data-missing" |
                     "kernel-data-missing" |
                     "kernel_param-data-missing" |
                     "ldconfig-data-missing" |
                     "libraries-data-missing" |
                     "lsb_tools-data-missing" |
                     "lscpu-data-missing" |
                     "lspci_verbose_data_missing" |
                     "lshw-data-missing" |
                     "lustre-data-missing" |
                     "memory-data-missing" |
                     "mount-data-missing" |
                     "mpi_internode-data-missing" |
                     "mpi_local-data-missing" |
                     "ntp-data-missing" |
                     "opa-data-missing" |
                     "perl-data-missing" |
                     "process-data-missing" |
                     "python-data-missing" |
                     "rpm-data-missing" |
                     "services-data-missing" |
                     "sgemm-data-missing" |
                     "shells-data-missing" |
                     "ssf_version-data-missing" |
                     "storage-data-missing" |
                     "stream-data-missing" |
                     "tcl-data-missing" |
                     "xp-modes-data-missing")
                   (state ~diagnosed) (suppressed FALSE))
  =>
  (send ?sign put-state diagnosed)
  (send ?diag put-confidence (combine-confidence (send ?diag get-confidence)
                                                 (send ?sign get-confidence)))
  (send ?diag put-severity (combine-severity (send ?diag get-severity)
                                             (send ?sign get-severity)))
  (slot-insert$ ?diag source 1 (send ?sign get-source)))

(defrule non-uniform-hardware-initial
  "If there are any signs for non-uniform hardware, create a
   non-uniform hardware diagnosis and mark the sign as diagnosed.
   This rule only fires for the first non-uniform hardware sign per
   node, i.e., when the diagnosis does not already exist.  Once the
   diagnosis exists, it should not be duplicated.  Thus, there is a
   corresponding rule, non-uniform-hardware-subsequent, for the case
   where there are multiple signs leading to this diagnosis."
  (object (is-a CONFIG) (name [config]))
  ?sign <- (object (is-a SIGN) (node_id ?node)
                   (id "cpu-model-name-not-uniform" |
                       "ethernet-firmware-version-is-not-consistent" |
                       "infiniband-ca-type-is-not-consistent" |
                       "infiniband-device-is-not-consistent" |
                       "infiniband-firmware-version-is-not-consistent" |
                       "infiniband-hardware-version-is-not-consistent" |
                       "infiniband-physical-state-is-not-consistent" |
                       "infiniband-physlot-is-not-consistent" |
                       "infiniband-rate-is-not-consistent" |
                       "infiniband-rev-is-not-consistent" |
                       "infiniband-state-is-not-consistent" |
                       "lshw-key-missing" |
                       "lshw-not-uniform" |
                       "memory-not-uniform" |
                       "opa-ca-is-not-consistent" |
                       "opa-device-is-not-consistent" |
                       "opa-firmware-version-is-not-consistent" |
                       "opa-hardware-version-is-not-consistent" |
                       "opa-physical-state-is-not-consistent" |
                       "opa-physlot-is-not-consistent" |
                       "opa-rate-is-not-consistent" |
                       "opa-state-is-not-consistent")
                   (state ~diagnosed) (suppressed FALSE))
  (not (object (is-a SHARED_DIAGNOSIS) (node_id ?node) (id "non-uniform-hardware")))
  =>
  (send ?sign put-state diagnosed)
  (make-instance of SHARED_DIAGNOSIS (node_id ?node)
                 (confidence (send ?sign get-confidence))
                 (severity (send ?sign get-severity))
                 (id "non-uniform-hardware")
                 (source (send ?sign get-source))))

(defrule non-uniform-hardware-subsequent
  "This rule is related to non-uniform-hardware-initial.  The
   difference is that this rule fires only after the initial diagnosis
   has already been created.  This rule marks the sign as diagnosed,
   and also adds to the list of signs that produced the diagnosis."
  (object (is-a CONFIG) (name [config]))
  ?diag <- (object (is-a SHARED_DIAGNOSIS) (node_id ?node) (id "non-uniform-hardware"))
  ?sign <- (object (is-a SIGN) (node_id ?node)
                   (id "cpu-model-name-not-uniform" |
                       "ethernet-firmware-version-is-not-consistent" |
                       "infiniband-ca-type-is-not-consistent" |
                       "infiniband-device-is-not-consistent" |
                       "infiniband-firmware-version-is-not-consistent" |
                       "infiniband-hardware-version-is-not-consistent" |
                       "infiniband-physical-state-is-not-consistent" |
                       "infiniband-physlot-is-not-consistent" |
                       "infiniband-rate-is-not-consistent" |
                       "infiniband-rev-is-not-consistent" |
                       "infiniband-state-is-not-consistent" |
                       "lshw-key-missing" |
                       "lshw-not-uniform" |
                       "memory-not-uniform" |
                       "opa-ca-is-not-consistent" |
                       "opa-device-is-not-consistent" |
                       "opa-firmware-version-is-not-consistent" |
                       "opa-hardware-version-is-not-consistent" |
                       "opa-physical-state-is-not-consistent" |
                       "opa-physlot-is-not-consistent" |
                       "opa-rate-is-not-consistent" |
                       "opa-state-is-not-consistent")
                   (state ~diagnosed) (suppressed FALSE))
  =>
  (send ?sign put-state diagnosed)
  (send ?diag put-confidence (combine-confidence (send ?diag get-confidence)
                                                 (send ?sign get-confidence)))
  (send ?diag put-severity (combine-severity (send ?diag get-severity)
                                             (send ?sign get-severity)))
  (slot-insert$ ?diag source 1 (send ?sign get-source)))

(defrule non-uniform-software-initial
  "If there are any signs for non-uniform software, create a
   non-uniform software diagnosis and mark the sign as diagnosed.
   This rule only fires for the first non-uniform software sign per
   node, i.e., when the diagnosis does not already exist.  Once the
   diagnosis exists, it should not be duplicated.  Thus, there is a
   corresponding rule, non-uniform-software-subsequent, for the case
   where there are multiple signs leading to this diagnosis."
  (object (is-a CONFIG) (name [config]))
  ?sign <- (object (is-a SIGN) (node_id ?node)
                   (id "environment-variable-not-uniform" |
                       "ethernet-driver-is-not-consistent" |
                       "ethernet-driver-version-is-not-consistent" |
                       "ethernet-mtu-is-not-consistent" |
                       "infiniband-driver-is-not-consistent" |
                       "infiniband-memlock-is-not-consistent" |
                       "infiniband-ofed-version-is-not-consistent" |
                       "ip-address-not-consistent" |
                       "kernel-not-uniform" |
                       "kernel-param-not-uniform" |
                       "mount-not-uniform-home-inode" |
                       "mount-not-uniform-home-path" |
                       "mpi-local-path-not-uniform" |
                       "opa-driver-is-not-consistent" |
                       "opa-memlock-is-not-consistent" |
                       "python-not-uniform" |
                       "perl-not-uniform" |
                       "rpm-is-missing" |
                       "rpm-is-extra" |
                       "tcl-not-uniform")
                   (state ~diagnosed) (suppressed FALSE))
  (not (object (is-a SHARED_DIAGNOSIS) (node_id ?node) (id "non-uniform-software")))
  =>
  (send ?sign put-state diagnosed)
  (make-instance of SHARED_DIAGNOSIS (node_id ?node)
                 (confidence (send ?sign get-confidence))
                 (severity (send ?sign get-severity))
                 (id "non-uniform-software")
                 (source (send ?sign get-source))))

(defrule non-uniform-software-subsequent
  "This rule is related to non-uniform-software-initial.  The
   difference is that this rule fires only after the initial diagnosis
   has already been created.  This rule marks the sign as diagnosed,
   and also adds to the list of signs that produced the diagnosis."
  (object (is-a CONFIG) (name [config]))
  ?diag <- (object (is-a SHARED_DIAGNOSIS) (node_id ?node) (id "non-uniform-software"))
  ?sign <- (object (is-a SIGN) (node_id ?node)
                   (id "environment-variable-not-uniform" |
                       "ethernet-driver-is-not-consistent" |
                       "ethernet-driver-version-is-not-consistent" |
                       "ethernet-mtu-is-not-consistent" |
                       "infiniband-driver-is-not-consistent" |
                       "infiniband-memlock-is-not-consistent" |
                       "infiniband-ofed-version-is-not-consistent" |
                       "ip-address-not-consistent" |
                       "kernel-not-uniform" |
                       "kernel-param-not-uniform" |
                       "mount-not-uniform-home-inode" |
                       "mount-not-uniform-home-path" |
                       "mpi-local-path-not-uniform" |
                       "opa-driver-is-not-consistent" |
                       "opa-memlock-is-not-consistent" |
                       "python-not-uniform" |
                       "perl-not-uniform" |
                       "rpm-is-missing" |
                       "rpm-is-extra" |
                       "tcl-not-uniform")
                   (state ~diagnosed) (suppressed FALSE))
  =>
  (send ?sign put-state diagnosed)
  (send ?diag put-confidence (combine-confidence (send ?diag get-confidence)
                                                 (send ?sign get-confidence)))
  (send ?diag put-severity (combine-severity (send ?diag get-severity)
                                             (send ?sign get-severity)))
  (slot-insert$ ?diag source 1 (send ?sign get-source)))

