; Copyright 2006-2019 Intel Corporation
;
; This software and the related documents are Intel copyrighted materials, and
; your use of them is governed by the express license under which they were
; provided to you (License). Unless the License provides otherwise, you may
; not use, modify, copy, publish, distribute, disclose, or transmit this
; software or the related documents without Intel's prior written permission.
;
; This software and the related documents are provided as is, with no express
; or implied warranties, other than those that are expressly stated in the
; License.

(defrule tcl-not-uniform
  "If the tcl version is not the same as at least 90% of the other nodes,
  then the node should be flagged as non-uniform.  The fewer other
  nodes that have the same tcl version increases the confidence
  that the node with the different version is incorrect."
  (object (is-a CONFIG) (name [config])
          (clck-checks $? tcl $?))
  ?obj <- (object (is-a TCL) (latest-timestamp TRUE) (node_id ?node)
                  (version ?ver&:(!= (str-length ?ver) 0))
                  (multiset_control TRUE))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj uniformity-key) ?ver) 0.9))
  =>
  (bind ?fraction (* 100 (send ?obj fraction (send ?obj uniformity-key) ?ver)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (- 100 ?fraction)) (severity 40) (source ?obj)
                 (state observed) (value TRUE) (id "tcl-not-uniform")
                 (args (create$ ?fraction ?ver))))

