#!/bin/bash
# Copyright 2015-2019 Intel Corporation
#
# This software and the related documents are Intel copyrighted materials, and
# your use of them is governed by the express license under which they were
# provided to you (License). Unless the License provides otherwise, you may
# not use, modify, copy, publish, distribute, disclose, or transmit this
# software or the related documents without Intel's prior written permission.
#
# This software and the related documents are provided as is, with no express
# or implied warranties, other than those that are expressly stated in the
# License.

UNAME_SYSTEM=`(uname -s) 2>/dev/null`  || UNAME_SYSTEM=unknown
ARCH=`uname -m`

case "${UNAME_SYSTEM}" in
Darwin)
	hash_algorithm="md5 -r"
	;;
Linux|GNU|GNU/*)
	hash_algorithm=md5sum
	;;
unknown)
	echo "Unrecognized system"
	exit 1
	;;
*)
	echo "Unable to detect system type"
	exit 2
	;;
esac

# TODO: change /usr/bin back to the whole local filesystem
case "${ARCH}" in
"x86_64")
	find /usr/bin -mount -type f -print0 | xargs -0 ${hash_algorithm}
;;
"k1om")
	find /usr/bin -xdev -type f | xargs ${hash_algorithm}
;;
*)
	echo "Error unknown arch. (${ARCH})"
	exit 3
;;
esac

