#!/bin/bash
# Copyright 2018-2019 Intel Corporation
#
# This software and the related documents are Intel copyrighted materials, and
# your use of them is governed by the express license under which they were
# provided to you (License). Unless the License provides otherwise, you may
# not use, modify, copy, publish, distribute, disclose, or transmit this
# software or the related documents without Intel's prior written permission.
#
# This software and the related documents are provided as is, with no express
# or implied warranties, other than those that are expressly stated in the
# License.
. $(dirname $0)/../../functions

no_gl_detected=70
invalid_ld_path=71
gl_detection_exit_code=$no_gl_detected
ld_path_exit_code=0

function detect_cxx_runtime() {
  # if a valid path is detected
  if [[ ! -z "$gl_path" ]]; then
    # loop over the available libstdc++
    for lpath in $gl_path
    do
      compatibiltity_list=$(strings "$lpath" |  sed -n 's/^GLIBCXX_\([0-9]\+.[0-9]\+\(.[0-9]\+\)\+\)$/\1/p' | sort -V 2>/dev/null)
      final_list=""
      if [[ ! -z "$compatibiltity_list" ]]; then
        for entry in $compatibiltity_list
        do
          final_list=$final_list"|"$entry
        done
        echo "libstdc++_path: " $lpath
        echo "runtime_compatibilties :" $final_list
        gl_detection_exit_code=0
        return
      fi
    done
  fi
  gl_detection_exit_code=$no_gl_detected
}

if [[ ! -z $LD_LIBRARY_PATH ]]; then
  # get the list of paths
  path_list=${LD_LIBRARY_PATH//:/ }
  for path in $path_list
  do
    gl_path=`find $path -name "libstdc++.so*" 2>/dev/null`
    detect_cxx_runtime
    if [[ $gl_detection_exit_code -eq 0 ]]; then
      break
    fi
  done
else
  _stderr "LD_LIBRARY_PATH environment variable not defined or null"
  ld_path_exit_code=$invalid_ld_path
fi

# either the LD_LIBRARY_PATH is not set or libGL is not found,
# either way use ldconfig
if [[ $gl_detection_exit_code -ne 0 ]]; then
  # check if user provided ldconfig_path
  if [[ ! -z $CLCK_PROVIDER_DETECT_GXX_INFO_LDCONFIG_PATH ]]; then
    ldconfig_path=$CLCK_PROVIDER_DETECT_GXX_INFO_LDCONFIG_PATH
  else
    # try detecting the ldconfig path
    get_tool_presence "ldconfig"
    ldconfig_path=$tool_path
    if [[ "$ldconfig_path" == "CLCK_LOCATION_NOT_FOUND" ]]; then
      _stderr "ldconfig command not found"
      exit 127
    fi
  fi
  ld_out=$($ldconfig_path -p)
  exit_code=$?
  if [[ ! -z "$ld_out" ]]; then
    gl_path_list=$(echo "$ld_out" | sed -n 's/\s*libstdc++\.so.*(.*,x86-64).*=> \(.*libstdc++.so.*\)/\1/p')
  fi
  # ldconfig can return multiple paths, so account for that
  for path in $gl_path_list
  do
    gl_path=$path
    detect_cxx_runtime
    if [[ $gl_detection_exit_code -eq 0 ]]; then
      break
    fi
  done
fi

if [[ $gl_detection_exit_code -ne 0 ]]; then
  if [[ $ld_path_exit_code -ne 0 ]]; then
    exit $ld_path_exit_code
  else
    exit $exit_code
  fi
fi
