#!/bin/bash
# Copyright 2017-2019 Intel Corporation
#
# This software and the related documents are Intel copyrighted materials, and
# your use of them is governed by the express license under which they were
# provided to you (License). Unless the License provides otherwise, you may
# not use, modify, copy, publish, distribute, disclose, or transmit this
# software or the related documents without Intel's prior written permission.
#
# This software and the related documents are provided as is, with no express
# or implied warranties, other than those that are expressly stated in the
# License.

. $(dirname $0)/../../functions

if [ ${UID} != "0" ]; then
    _stderr "Root access is required for dmidecode to run properly."
    exit 1
fi

get_tool_presence "dmidecode"

if [[ "$CLCK_PROVIDER_DMIDECODE_PATH" == "" ]]; then
    DMIDECODE_BINARY=${tool_path}
else
    DMIDECODE_BINARY="${CLCK_PROVIDER_DMIDECODE_PATH}/dmidecode"
fi

#Data labels used by dmidecode that we are interested in.
MEM_TAGS="Memory Device|Size|Form Factor|Locator|Bank Locator|Manufacturer|Speed|Type"
BOARD_TAGS="Base Board Information|Manufacturer|Product Name|Version|Serial Number|Type"
# Run dmidecode but ensure the output is structured such that
# parsing of the output is more consistant.
echo "#####FORMATED_DMIDECODE_MEMORY_BEGIN#####"
${DMIDECODE_BINARY} -t 17 | grep -E "^\s*(${MEM_TAGS})"
echo "#####FORMATED_DMIDECODE_MEMORY_END#####"

echo "#####FORMATED_DMIDECODE_BASEBOARD_BEGIN#####"
${DMIDECODE_BINARY} -t 2 | grep -E "^\s*(${BOARD_TAGS})"
echo "#####FORMATED_DMIDECODE_BASEBOARD_END#####"


