#!/bin/bash
# Copyright 2018-2019 Intel Corporation
#
# This software and the related documents are Intel copyrighted materials, and
# your use of them is governed by the express license under which they were
# provided to you (License). Unless the License provides otherwise, you may
# not use, modify, copy, publish, distribute, disclose, or transmit this
# software or the related documents without Intel's prior written permission.
#
# This software and the related documents are provided as is, with no express
# or implied warranties, other than those that are expressly stated in the
# License.

# Path to all ethernet devices
SYS_EN_PATH=/sys/class/net

# Check if there is a ethernet device folder
if [ ! -d "$SYS_EN_PATH" ]; then
  exit 1
fi

# For all ethernet devices
for EN_DEVICE in $(ls $SYS_EN_PATH)
do

  EN_DEVICE_PATH="${SYS_EN_PATH}/${EN_DEVICE}"

  # Check if its an ethernet device
  if [ ! $(cat ${EN_DEVICE_PATH}/mtu) == 1500 ]; then
    continue
  fi

  # Get ethernet device id
  EN_DEVICE_ID="${EN_DEVICE_PATH}/device/device"
  if [ ! -f ${EN_DEVICE_ID} ]; then
    continue
  fi
  echo EN_DEVICE_ID
  cat $EN_DEVICE_ID

  # Get ethernet device vendor
  EN_DEVICE_VENDOR="${EN_DEVICE_PATH}/device/vendor"
  if [ -f ${EN_DEVICE_VENDOR} ]; then
    echo EN_DEVICE_VENDOR
    cat $EN_DEVICE_VENDOR
  fi

  # Get label of ethernet device
  EN_DEVICE_LABEL="${EN_DEVICE_PATH}/device/label"
  if [ -f ${EN_DEVICE_LABEL} ]; then
    echo EN_DEVICE_LABEL
    cat $EN_DEVICE_LABEL
  fi

  # Get state of ethernet device
  EN_DEVICE_STATE="${EN_DEVICE_PATH}/operstate"
  if [ -f ${EN_DEVICE_STATE} ]; then
    echo EN_DEVICE_STATE
    cat $EN_DEVICE_STATE
  fi

  # Get speed of ethernet device
  EN_DEVICE_SPEED="${EN_DEVICE_PATH}/speed"
  if [ -f ${EN_DEVICE_SPEED} ]; then
    echo EN_DEVICE_SPEED
    cat $EN_DEVICE_SPEED
  fi
done
