#!/bin/bash
# Copyright 2017-2019 Intel Corporation
#
# This software and the related documents are Intel copyrighted materials, and
# your use of them is governed by the express license under which they were
# provided to you (License). Unless the License provides otherwise, you may
# not use, modify, copy, publish, distribute, disclose, or transmit this
# software or the related documents without Intel's prior written permission.
#
# This software and the related documents are provided as is, with no express
# or implied warranties, other than those that are expressly stated in the
# License.

. $(dirname $0)/../../functions

# check script parameteres to make sure nothing is critically wrong
function check {
    if [ ! -f $1 ]; then
        \echo "\"$1\" does not exist" >&2
        exit 1
    elif [[ "$1" == "" ]]; then
        \echo "file \"$2\" not provided" >&2
        exit 2
    elif [[ ! "$1" =~ "$2" ]]; then
        \echo "file \"$1\" may not be used for file \"$2\"" >&2
        exit 3
    else
        return 0
    fi
}


# INPUT: filename
# OUTPUT: filename,perms,user,group,md5sum
function file_info {

    if \stat $1 &>/dev/null; then
        P=$(\stat --printf='%A' $1)
        O=$(\stat --printf='%U' $1)
        G=$(\stat --printf='%G' $1)
        STAT_AVAILABLE=1
    else
        P=""
        O=""
        G=""
        STAT_AVAILABLE=0
    fi

    if \md5sum $1 &>/dev/null; then
        M=$(\md5sum $1 | \awk '{print $1}')
        MD5SUM_AVAILABLE=1
    else
        M=""
        MD5SUM_AVAILABLE=0
    fi

    # file, perms, owner, group, md5sum
    OUTPUT="$1,$P,$O,$G,$M"
    if [ $STAT_AVAILABLE -eq 1 ] && [ $MD5SUM_AVAILABLE -eq 1 ]; then
        \echo $OUTPUT >&1
    else
        _stderr $OUTPUT
    fi

}


# main function
function main {

    check $1 "/white";  white=$1
    check $2 "/black";  black=$2

    # blacklists are interpreted as path regular expressions
    for file in $(\rpm --query --all --configfiles | \grep -v "^[^/]" | \
      \grep -vGf $black); do
        file_info $file
    done

    # whitelists are interpreted as absolute file paths
    while read file; do
        file_info $file
    done < $white

}


main $@
