#!/bin/bash
# Copyright 2017-2019 Intel Corporation
#
# This software and the related documents are Intel copyrighted materials, and
# your use of them is governed by the express license under which they were
# provided to you (License). Unless the License provides otherwise, you may
# not use, modify, copy, publish, distribute, disclose, or transmit this
# software or the related documents without Intel's prior written permission.
#
# This software and the related documents are provided as is, with no express
# or implied warranties, other than those that are expressly stated in the
# License.
. $(dirname $0)/../../functions

if [ ! -f "${CLCK_DP_NODEFILE}" ] ; then
    echo "Missing or invalid nodefile: '${CLCK_DP_NODEFILE}'" >&2
    exit 1
fi

# setup mpi environment
mpivars
mklvars

NNODES=`cat $CLCK_DP_NODEFILE | wc -l`
LTMPDIR=`mktemp -d --tmpdir hpcg.XXXXXX`
XHPCG="$(dirname $0)/../common/bin/intel64/xhpcg_avx2"

# get the minimum number of cores per node
num_physical_cores_cluster_min $CLCK_DP_NODEFILE $(dirname $0)

# get the minimum number of sockets
num_sockets_cluster_min $CLCK_DP_NODEFILE $(dirname $0)

# Set ppn 
# Intel(R) Xeon(R) processor or Intel(R) Xeon Phi(TM) processor
ppn=${num_sockets_cluster_min}
if [[ ! -z $CLCK_PROVIDER_HPCG_CLUSTER_PPN ]] ; then
    if [[ $CLCK_PROVIDER_HPCG_CLUSTER_PPN =~ ^[0-9]+$ && $CLCK_PROVIDER_HPCG_CLUSTER_PPN -gt 0 ]] ; then
        ppn=${CLCK_PROVIDER_HPCG_CLUSTER_PPN}
    elif [[ $CLCK_PROVIDER_HPCG_CLUSTER_PPN == "core" ]] ; then
        ppn=${num_physical_cores_cluster_min}
    fi
fi


# Configure MPI to timeout before CLCK does to give MPI a chance to cleanup
if [[ ! -z $CLCK_TIMEOUT && $CLCK_TIMEOUT -gt 5 ]]; then
    export I_MPI_JOB_TIMEOUT=$(( $CLCK_TIMEOUT - 5 ))
fi

# Configure MPI to cleanup
export I_MPI_MPIRUN_CLEANUP=1

# Look for user configured options sent in via environment variables

if [ "$CLCK_PROVIDER_HPCG_CLUSTER_FABRICS" == "" ]; then
    MY_FABRICS=""
else
    MY_FABRICS="-genv I_MPI_FABRICS=${CLCK_PROVIDER_HPCG_CLUSTER_FABRICS}"
fi

if [ "$CLCK_PROVIDER_HPCG_CLUSTER_DAPL_PROVIDER" == "" ]; then
    MY_DAPL=""
else
    MY_DAPL="-genv I_MPI_DAPL_PROVIDER=${CLCK_PROVIDER_HPCG_CLUSTER_DAPL_PROVIDER}"
fi

if [ "$CLCK_PROVIDER_HPCG_CLUSTER_DAPL_PROVIDER_LIST" == "" ]; then
    MY_DAPL_LIST=""
else
    MY_DAPL_LIST="-genv I_MPI_DAPL_PROVIDER_LIST=${CLCK_PROVIDER_HPCG_CLUSTER_DAPL_PROVIDER_LIST}"
fi

if [ "$CLCK_PROVIDER_HPCG_CLUSTER_OFA_ADAPTER_NAME" == "" ]; then
    MY_OFA=""
else
    MY_OFA="-genv I_MPI_OFA_ADAPTER_NAME=${CLCK_PROVIDER_HPCG_CLUSTER_OFA_ADAPTER_NAME}"
fi

if [ "$CLCK_PROVIDER_HPCG_CLUSTER_OFI_PROVIDER" == "" ]; then
    MY_OFI=""
else
    MY_OFI="-genv I_MPI_OFI_PROVIDER=${CLCK_PROVIDER_HPCG_CLUSTER_OFI_PROVIDER}"
fi

if [ "$CLCK_PROVIDER_HPCG_CLUSTER_TCP_NETMASK" == "" ]; then
    MY_TCP=""
else
    MY_TCP="-genv I_MPI_TCP_NETMASK=${CLCK_PROVIDER_HPCG_CLUSTER_TCP_NETMASK}"
fi

if [ "$CLCK_PROVIDER_HPCG_CLUSTER_TMI_PROVIDER" == "" ]; then
    MY_TMI=""
else
    MY_TMI="-genv I_MPI_TMI_PROVIDER=${CLCK_PROVIDER_HPCG_CLUSTER_TMI_PROVIDER}"
fi

if [ "$CLCK_PROVIDER_HPCG_CLUSTER_OPTIONS" == "" ]; then
    MY_OPTIONS=""
else
    MY_OPTIONS=${CLCK_PROVIDER_HPCG_CLUSTER_OPTIONS}
fi

if [ "$CLCK_PROVIDER_HPCG_CLUSTER_MPI_PIN" = "" ]; then
    MY_MPI_PIN=""
else
    MY_MPI_PIN="-genv I_MPI_PIN ${CLCK_PROVIDER_HPCG_CLUSTER_MPI_PIN}"
fi

if [ "$CLCK_PROVIDER_HPCG_CLUSTER_KMP_AFFINITY" = "" ]; then
    MY_KMP_AFFINITY="compact,granularity=fine,1,0"
else
    MY_KMP_AFFINITY="${CLCK_PROVIDER_HPCG_CLUSTER_KMP_AFFINITY}"
fi

if [ "$CLCK_PROVIDER_HPCG_CLUSTER_OMP_NUM_THREADS" = "" ]; then
    let MY_OMP_NUM_THREADS=num_physical_cores_cluster_min/num_sockets_cluster_min
else
    MY_OMP_NUM_THREADS="${CLCK_PROVIDER_HPCG_CLUSTER_OMP_NUM_THREADS}"
fi

if [ "$CLCK_PROVIDER_HPCG_CLUSTER_N" = "" ]; then
    MY_N="160"
else
    if [[ "${CLCK_PROVIDER_HPCG_CLUSTER_N}" =~ ^[0-9]+$ && ${CLCK_PROVIDER_HPCG_CLUSTER_N} -gt 0 ]] ; then
      MY_N=${CLCK_PROVIDER_HPCG_CLUSTER_N}
    else
      MY_N="160"
    fi
fi
if [ "$CLCK_PROVIDER_HPCG_CLUSTER_T" = "" ]; then
    MY_T="180"
else
    if [[ "${CLCK_PROVIDER_HPCG_CLUSTER_T}" =~ ^[0-9]+$ && ${CLCK_PROVIDER_HPCG_CLUSTER_T} -gt 0 ]] ; then
      MY_T=${CLCK_PROVIDER_HPCG_CLUSTER_T}
    else
      MY_T="180"
    fi
fi



let num_jobs=ppn*NNODES

# run HPCG
# If the user options has a I_MPI_DEBUG we over ride it with 5 anyway
#
echo "OMP_NUM_THREADS=$MY_OMP_NUM_THREADS KMP_AFFINITY=$MY_KMP_AFFINITY mpirun $MY_OPTIONS $MY_OFI $MY_OFA $MY_DAPL $MY_TCP $MY_TMI $MY_FABRICS $MY_MPI_PIN -genv OMP_NUM_THREADS=$MY_OMP_NUM_THREADS -genv KMP_AFFINITY=$MY_KMP_AFFINITY -genv I_MPI_DEBUG 12  -n $num_jobs -ppn $ppn -f $CLCK_DP_NODEFILE -gwdir $LTMPDIR $XHPCG -n${MY_N} -t${MY_T}"

time OMP_NUM_THREADS=$MY_OMP_NUM_THREADS KMP_AFFINITY=$MY_KMP_AFFINITY mpirun $MY_OPTIONS $MY_OFI $MY_OFA $MY_DAPL $MY_TCP $MY_TMI $MY_FABRICS $MY_MPI_PIN -genv OMP_NUM_THREADS=$MY_OMP_NUM_THREADS -genv KMP_AFFINITY=$MY_KMP_AFFINITY -genv I_MPI_DEBUG 12  -n $num_jobs -ppn $ppn -f $CLCK_DP_NODEFILE -gwdir $LTMPDIR $XHPCG -n${MY_N} -t${MY_T}

echo "" 
echo "===========================================" 
echo "" 
cat $LTMPDIR/hpcg_log_*txt
echo "" 
echo "===========================================" 
echo "" 
cat $LTMPDIR/n${MY_N}*yaml
echo "" 
echo "===========================================" 
echo "" 

# cleanup
if [ -d $LTMPDIR ] ; then
    rm -rf $LTMPDIR
fi
