#!/bin/bash
# Copyright 2015-2019 Intel Corporation
#
# This software and the related documents are Intel copyrighted materials, and
# your use of them is governed by the express license under which they were
# provided to you (License). Unless the License provides otherwise, you may
# not use, modify, copy, publish, distribute, disclose, or transmit this
# software or the related documents without Intel's prior written permission.
#
# This software and the related documents are provided as is, with no express
# or implied warranties, other than those that are expressly stated in the
# License.

if [[ "$CLCK_PROVIDER_LUSTRE_STRIPE_BINARY" == "" ]]; then
    LFS=/usr/bin/lfs
else
    LFS="${CLCK_PROVIDER_LUSTRE_STRIPE_BINARY}"
fi

if [ ! -x $LFS ]
then
    echo "error: can not find lustre executable: " $LFS
    exit 1
fi

. $(dirname $0)/../../functions

function run_test {
    fs=$1
    num_osts=$2
    mount_point=$3

    echo -n "NEW TEST, filesystem, $fs , num_osts, $num_osts"
    echo ", mount_point, $mount_point"

    #create unique file on cluster which sets the striping parameters to use
    # all of the osts up to the system limit (160)
    test_file=${mount_point}/.clck3_lustre_stripe_test_$(hostname)_$(date +%s%N)
    $LFS setstripe $test_file --stripe-size 0 -c -1 -i -1
    if [ $? -ne 0 ]
    then
	    echo "error creating test file " $test_file
	    rm -f $test_file
	    return
    fi

    #write file to disk
    dd if=/dev/zero of=$test_file bs=10485760 count=1 &> /dev/null
    if [ $? -ne 0 ]
    then
	    echo "error writing to test file " $test_file
	    rm -f $test_file
	    return
    fi

    #get string information
    $LFS getstripe $test_file
    rm -f $test_file
}

get_lustre_filesystems $LFS

for fs_idx in "${!fs_list[@]}"
do
    run_test ${fs_list[$fs_idx]} ${num_ost_list[$fs_idx]} ${mount_points[$fs_idx]}
done
