#!/bin/bash
# Copyright 2015-2019 Intel Corporation
#
# This software and the related documents are Intel copyrighted materials, and
# your use of them is governed by the express license under which they were
# provided to you (License). Unless the License provides otherwise, you may
# not use, modify, copy, publish, distribute, disclose, or transmit this
# software or the related documents without Intel's prior written permission.
#
# This software and the related documents are provided as is, with no express
# or implied warranties, other than those that are expressly stated in the
# License.

ARCH=`uname -m`
DIR=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )

. $(dirname $0)/../../functions

mpivars

\which mpirun

# This environment variable will be defined for us
# CLCK_DP_NODEFILE
if [ ! -f "${CLCK_DP_NODEFILE}" ] ; then
    echo "Missing or invalid nodefile: '${CLCK_DP_NODEFILE}'" >&2
    exit 1
fi

# Configure MPI to timeout before CLCK does to give MPI a chance to cleanup
if [[ ! -z $CLCK_TIMEOUT && $CLCK_TIMEOUT -gt 5 ]]; then
  export I_MPI_JOB_TIMEOUT=$(( $CLCK_TIMEOUT - 5 ))
fi

# Configure MPI to cleanup
export I_MPI_MPIRUN_CLEANUP=1

# Look for user configured options sent in via environment variables

if [ "$CLCK_PROVIDER_MPI_INTERNODE_FABRICS" == "" ]; then
    MY_FABRICS=""
else
    MY_FABRICS="-genv I_MPI_FABRICS=${CLCK_PROVIDER_MPI_INTERNODE_FABRICS}"
fi

if [ "$CLCK_PROVIDER_MPI_INTERNODE_DAPL_PROVIDER" == "" ]; then
    MY_DAPL=""
else
    MY_DAPL="-genv I_MPI_DAPL_PROVIDER=${CLCK_PROVIDER_MPI_INTERNODE_DAPL_PROVIDER}"
fi

if [ "$CLCK_PROVIDER_MPI_INTERNODE_DAPL_PROVIDER_LIST" == "" ]; then
    MY_DAPL_LIST=""
else
    MY_DAPL_LIST="-genv I_MPI_DAPL_PROVIDER_LIST=${CLCK_PROVIDER_MPI_INTERNODE_DAPL_PROVIDER_LIST}"
fi

if [ "$CLCK_PROVIDER_MPI_INTERNODE_OFA_ADAPTER_NAME" == "" ]; then
    MY_OFA=""
else
    MY_OFA="-genv I_MPI_OFA_ADAPTER_NAME=${CLCK_PROVIDER_MPI_INTERNODE_OFA_ADAPTER_NAME}"
fi

if [ "$CLCK_PROVIDER_MPI_INTERNODE_TCP_NETMASK" == "" ]; then
    MY_TCP=""
else
    MY_TCP="-genv I_MPI_TCP_NETMASK=${CLCK_PROVIDER_MPI_INTERNODE_TCP_NETMASK}"
fi

if [ "$CLCK_PROVIDER_MPI_INTERNODE_TMI_PROVIDER" == "" ]; then
    MY_TMI=""
else
    MY_TMI="-genv I_MPI_TMI_PROVIDER=${CLCK_PROVIDER_MPI_INTERNODE_TMI_PROVIDER}"
fi

if [ "$CLCK_PROVIDER_MPI_INTERNODE_OPTIONS" == "" ]; then
    MY_OPTIONS=""
else
    MY_OPTIONS=${CLCK_PROVIDER_MPI_INTERNODE_OPTIONS}
fi

if [ "$CLCK_PROVIDER_MPI_INTERNODE_MPI_PIN" = "" ]; then
    MY_MPI_PIN=""
else
    MY_MPI_PIN="-genv I_MPI_PIN ${CLCK_PROVIDER_MPI_INTERNODE_MPI_PIN}"
fi

# Note: If the user options has a I_MPI_DEBUG, we over ride it with 5 anyway
mpirun $MY_OPTIONS $MY_DAPL $MY_DAPL_LIST $MY_OFA $MY_TCP $MY_TMI $MY_FABRICS $MY_MPI_PIN -f ${CLCK_DP_NODEFILE} -ppn 1 -genv I_MPI_DEBUG 5 ${DIR}/mpi_hello_world
