#!/bin/bash
# Copyright 2015-2019 Intel Corporation
#
# This software and the related documents are Intel copyrighted materials, and
# your use of them is governed by the express license under which they were
# provided to you (License). Unless the License provides otherwise, you may
# not use, modify, copy, publish, distribute, disclose, or transmit this
# software or the related documents without Intel's prior written permission.
#
# This software and the related documents are provided as is, with no express
# or implied warranties, other than those that are expressly stated in the
# License.

ARCH=`uname -m`

. $(dirname $0)/../../functions

mpivars

\which mpirun

RESULT=$?

case "${ARCH}" in

    "x86_64")
    BIN="$(dirname $0)/intel64/mpi_local.impi"
    ;;
    *)
    exit 1
    ;;
esac

# Note: If the above command resulted in a value, we have a location
# for mpirun. So we can try and run mpi_local. If not, the STDOUT
# for this provider shall just remain empty.

# Configure MPI to timeout before CLCK does to give MPI a chance to cleanup
if [[ ! -z $CLCK_TIMEOUT && $CLCK_TIMEOUT -gt 5 ]]; then
    export I_MPI_JOB_TIMEOUT=$(( $CLCK_TIMEOUT - 5 ))
fi

if [ "$CLCK_PROVIDER_MPI_LOCAL_MPI_PIN" = "" ]; then
    MY_MPI_PIN=""
else
    MY_MPI_PIN="-genv I_MPI_PIN ${CLCK_PROVIDER_MPI_LOCAL_MPI_PIN}"
fi

if [ "$RESULT" == "0" ]
then
    # Note: The first line in STDOUT will be the path to mpirun.
    # Subsequently, there will be 40 asterixes. The analyze extension
    # transform plugin should look for the first occurance of
    # 40 asterixes to locate the point beyond which the output
    # of mpirun is present in the STDOUT.
    echo "****************************************"

    mpirun -genv I_MPI_DEBUG 5 $MY_MPI_PIN -n 4 $BIN
fi
