#!/bin/bash
# Copyright 2018-2019 Intel Corporation
#
# This software and the related documents are Intel copyrighted materials, and
# your use of them is governed by the express license under which they were
# provided to you (License). Unless the License provides otherwise, you may
# not use, modify, copy, publish, distribute, disclose, or transmit this
# software or the related documents without Intel's prior written permission.
#
# This software and the related documents are provided as is, with no express
# or implied warranties, other than those that are expressly stated in the
# License.

# Path to all nvme devices
SYS_NVME_PATH=/sys/class/nvme

# Check if there is a NVMe device folder
if [ ! -d "$SYS_NVME_PATH" ]; then
  exit 1
fi

# For all NVMe devices
for NVME_DEVICE in $(ls $SYS_NVME_PATH)
do
  # Path to information on a specific NVMe device
  NVME_DEVICE_PATH="${SYS_NVME_PATH}/${NVME_DEVICE}"

  # Get NVMe device id
  NVME_DEVICE_ID="${NVME_DEVICE_PATH}/device/device"
  if [ ! -f ${NVME_DEVICE_ID} ]; then
    continue
  fi
  echo NVME_DEVICE_ID
  cat $NVME_DEVICE_ID

  # Get NVMe device vendor
  NVME_DEVICE_VENDOR="${NVME_DEVICE_PATH}/device/vendor"
  if [ -f ${NVME_DEVICE_VENDOR} ]; then
    echo NVME_DEVICE_VENDOR
    cat $NVME_DEVICE_VENDOR
  fi

  # Get NVMe device model
  NVME_DEVICE_MODEL="${NVME_DEVICE_PATH}/model"
  if [ -f ${NVME_DEVICE_MODEL} ]; then
    echo NVME_DEVICE_MODEL
    cat $NVME_DEVICE_MODEL
  fi

  # Get NVMe firmware
  NVME_DEVICE_FW="${NVME_DEVICE_PATH}/firmware_rev"
  if [ -f ${NVME_DEVICE_FW} ]; then
    echo NVME_DEVICE_FW
    cat $NVME_DEVICE_FW
  fi

  # Get NVMe device state
  NVME_DEVICE_STATE="${NVME_DEVICE_PATH}/state"
  if [ -f ${NVME_DEVICE_STATE} ]; then
    echo NVME_DEVICE_STATE
    cat $NVME_DEVICE_STATE
  fi

  # Get NVMe device size
  NVME_DEVICE_CAPACITY=$(find $NVME_DEVICE_PATH/ -maxdepth 1 -type d -name nvme*n* -print)
  NVME_DEVICE_CAPACITY="${NVME_DEVICE_CAPACITY}/size"
  if [ -f ${NVME_DEVICE_CAPACITY} ]; then
    echo NVME_DEVICE_CAPACITY
    cat $NVME_DEVICE_CAPACITY
  fi
done
