#!/bin/bash
# Copyright 2015-2019 Intel Corporation
#
# This software and the related documents are Intel copyrighted materials, and
# your use of them is governed by the express license under which they were
# provided to you (License). Unless the License provides otherwise, you may
# not use, modify, copy, publish, distribute, disclose, or transmit this
# software or the related documents without Intel's prior written permission.
#
# This software and the related documents are provided as is, with no express
# or implied warranties, other than those that are expressly stated in the
# License.

. $(dirname $0)/../../functions


if [[ "$CLCK_PROVIDER_OPASMAQUERY_PATH" == "" ]]; then
    get_tool_path "opasmaquery"
    SMA_BINARY=$tool_path
else
    SMA_BINARY="${CLCK_PROVIDER_OPASMAQUERY_PATH}/opasmaquery"
fi

if [[ "$CLCK_PROVIDER_OPASMAQUERY_LSPCI_PATH" == "" ]]; then
    get_tool_path "lspci"
    LSPCI_BIN="$tool_path"
else
    LCPSI_BIN="${CLCK_PROVIDER_OPASMAQUERY_LSPCI_PATH}"
fi

dev_present_ec=72
lspci_out=`$LSPCI_BIN -k -D -v -nn`
echo $lspci_out | \grep "8086:24f0" >& /dev/null
if [[ $? == 0 ]]; then
  dev_present_ec=0
else
  if [[ $dev_present_ec -ne 0 ]]; then
    # try 2f41
    echo $lspci_out | \grep "8086:24f1" >& /dev/null
    if [[ $? == 0 ]]; then
      # found it, sane to go forward
      dev_present_ec=0
    fi
  fi
fi

# Use opainfo to get the number of hfi seen by the system.
# The system sees different ports as seperate HFI devices
# If they are from the same card they will have an identical
# SystemImage.


# Run opasmaquery  using -h to force to default hfi port
# and the -p lets us cycle through all available ports on
# the system.
# HFI's and with -g to force flag output.
i=1
res=0

exit_code=70

while [ $res -eq 0 ]; do
    ${SMA_BINARY} -g -h 0 -p $i -o nodeinfo > /dev/null 2>&1
    res=$?
    if  [ $res -eq 0 ]; then
        echo "####LOOKING AT PORT $i####"
        ${SMA_BINARY} -g -h 0 -p $i -o nodeinfo
        ${SMA_BINARY} -g -h 0 -p $i -o portinfo
        echo "####END PORT $i####"
        output=`${SMA_BINARY} -g -h 0 -p $i -o portinfo`
        main_exit_code=$?
        if [[ main_exit_code -eq 0 && exit_code -ne 0 ]]; then
          port_state_exit_code=70
          filtered_op=`echo $output | grep "PortStates.s.PortState"`
          if [[ $? -eq 0 ]]; then
            echo $filtered_op | grep "0x4" > /dev/null 2>&1
            if [[ $? -eq 0 ]]; then
               port_state_exit_code=0
            fi
          fi
          phy_state_exit_code=70
          filtered_op_phys=`echo $output | grep "PortStates.s.PortPhysicalState"`
          if [[ $? -eq 0 ]]; then
            echo $filtered_op_phys | grep "0x5" > /dev/null 2>&1
            if [[ $? -eq 0 ]]; then
               phy_state_exit_code=0
            fi
          fi
          if [[ $phy_state_exit_code -eq 0 && $port_state_exit_code -eq 0 ]]; then
            exit_code=0
          fi
        fi
        i=$((i + 1))
    else
        # atleast one port up
        if [[ $exit_code == 0 ]]; then
          exit 0
        else
          # none of the ports up, see if devices present
          if [[ $dev_present_ec == 0 ]]; then
            if [[ $main_exit_code -eq 0 ]]; then
              exit $exit_code
            else
              exit 71
            fi
          else
            exit 72
          fi
        fi
    fi
done


if [[ $dev_present_ec == 0 ]]; then
  exit $exit_code
else
  exit $dev_present_ec
fi
