#!/bin/bash
# Copyright 2015-2019 Intel Corporation
#
# This software and the related documents are Intel copyrighted materials, and
# your use of them is governed by the express license under which they were
# provided to you (License). Unless the License provides otherwise, you may
# not use, modify, copy, publish, distribute, disclose, or transmit this
# software or the related documents without Intel's prior written permission.
#
# This software and the related documents are provided as is, with no express
# or implied warranties, other than those that are expressly stated in the
# License.

. $(dirname $0)/../../functions

ARCH=`uname -m`

# set the number of iterations for sgemm wrapper
function set_clck_iterations() {
    clck_sgemm_iterations=$1
    if [[ ! -z $CLCK_PROVIDER_SGEMM_ITERATIONS ]] ; then
        if [[ $CLCK_PROVIDER_SGEMM_ITERATIONS =~ ^[0-9]+$ && $CLCK_PROVIDER_SGEMM_ITERATIONS -gt 0 ]] ; then
            clck_sgemm_iterations=$CLCK_PROVIDER_SGEMM_ITERATIONS
        fi
    fi
}

# sets the memory usage parameters
function set_clck_memory_usage() {
    clck_mu_rtype=1
    clck_sgemm_memory_usage=$1
    clck_k_param=$2
    if [[ ! -z $CLCK_PROVIDER_SGEMM_MEMORY_USAGE && \
               $CLCK_PROVIDER_SGEMM_MEMORY_USAGE =~ ^[0-9]+$ ]] ; then
        if [[ $CLCK_PROVIDER_SGEMM_MEMORY_USAGE -gt 0 && \
            $CLCK_PROVIDER_SGEMM_MEMORY_USAGE -le 100 ]] ; then
            clck_sgemm_memory_usage=${CLCK_PROVIDER_SGEMM_MEMORY_USAGE}
            clck_mu_rtype=0
            if [[ ! -z $CLCK_PROVIDER_SGEMM_K_PARAMETER ]] ; then
                 if [[ $CLCK_PROVIDER_SGEMM_K_PARAMETER =~ ^[0-9]+$ && \
                       $CLCK_PROVIDER_SGEMM_K_PARAMETER -gt 0 ]] ; then
                     clck_k_param=${CLCK_PROVIDER_SGEMM_K_PARAMETER}
                     clck_mu_rtype=0
                 else
                     clck_mu_rtype=2
                     _stderr "invalid k parameter , using the default value"
                 fi
            fi
            return $clck_mu_rtype
        else
            _stderr "invalid memory usage percentage, using the default value"
        fi
    fi
    return $clck_mu_rtype
}

function set_clck_m_n_k() {
    clck_m_param=$1
    clck_n_param=$2
    clck_k_param=$3

    # memory percentage was set, do not over-ride
    if [[ $clck_mu_rtype == 0 ]] ; then
        return;
    fi

    if [[ ! -z $CLCK_PROVIDER_SGEMM_M_PARAMETER && ! -z $CLCK_PROVIDER_SGEMM_N_PARAMETER && ! -z $CLCK_PROVIDER_SGEMM_K_PARAMETER  ]] ; then
          if [[ ( $CLCK_PROVIDER_SGEMM_M_PARAMETER =~ ^[0-9]+$ && $CLCK_PROVIDER_SGEMM_M_PARAMETER -gt 0 ) && \
              ( $CLCK_PROVIDER_SGEMM_N_PARAMETER =~ ^[0-9]+$ && $CLCK_PROVIDER_SGEMM_N_PARAMETER -gt 0 ) && \
               ( $CLCK_PROVIDER_SGEMM_K_PARAMETER =~ ^[0-9]+$ && $CLCK_PROVIDER_SGEMM_K_PARAMETER -gt 0 ) ]] ; then
            clck_m_param=${CLCK_PROVIDER_SGEMM_M_PARAMETER}
            clck_n_param=${CLCK_PROVIDER_SGEMM_N_PARAMETER}
            clck_k_param=${CLCK_PROVIDER_SGEMM_K_PARAMETER}
        fi
    fi
}

function set_clck_kmp_affinity() {
    if [[ "$CLCK_PROVIDER_SGEMM_KMP_AFFINITY" == "" ]]; then
        export KMP_AFFINITY="$1"
    else
        export KMP_AFFINITY="${CLCK_PROVIDER_SGEMM_KMP_AFFINITY}"
    fi
}

function set_clck_kmp_hw_subset() {
    if [[ "$CLCK_PROVIDER_SGEMM_KMP_HW_SUBSET" == "" ]]; then
        export KMP_HW_SUBSET="$1"
    else
        export KMP_HW_SUBSET="${CLCK_PROVIDER_SGEMM_KMP_HW_SUBSET}"
    fi
}

function get_clck_task_set() {
    local clck_taskset_threads=$(( $1*$2 ))
    clck_taskset_threads=$(( clck_taskset_threads-1 ))
    clck_taskset_list="${clck_taskset_threads}"
    clck_taskset_list="0-${clck_taskset_list}"
    if [[ "$CLCK_PROVIDER_SGEMM_TASKSET" != "" ]]; then
        clck_taskset="${CLCK_PROVIDER_SGEMM_TASKSET}"
    fi
}

function clck_detect_numactl() {
    get_tool_presence "numactl"
    clck_numactl_bin=$tool_path
    if [[ "$CLCK_PROVIDER_SGEMM_NUMACTL_BINARY" != "" ]]; then
        clck_numactl_bin="${CLCK_PROVIDER_SGEMM_NUMACTL_BINARY}"
    fi
    if [[ $clck_numactl_bin == "CLCK_LOCATION_NOT_FOUND" ]]; then
        clck_numactl_bin="";
    else
        clck_numactl_bin="$clck_numactl_bin -i all"
    fi
}

function clck_detect_taskset() {
    get_tool_presence "taskset"
    clck_taskset_bin=$tool_path
    if [[ "$CLCK_PROVIDER_SGEMM_TASKSET_BINARY" != "" ]]; then
        clck_taskset_bin="${CLCK_PROVIDER_SGEMM_TASKSET_BINARY}"
    fi
    # number of supported cores and threads/core
    num_physical_cores
    num_threads_per_core
    get_clck_task_set ${num_physical_cores} ${num_threads_per_core}
    if [[ $clck_taskset_bin == "CLCK_LOCATION_NOT_FOUND" ]]; then
        clck_taskset_bin="";
    else
        clck_taskset_bin="$clck_taskset_bin -c ${clck_taskset_list}"
    fi
}

function clck_set_hbm_limit() {
    clck_sgemm_hbm_limit=$1
    if [[ ! -z $CLCK_PROVIDER_SGEMM_FAST_MEMORY_LIMIT ]] ; then
        if [[ $CLCK_PROVIDER_SGEMM_FAST_MEMORY_LIMIT =~ ^[0-9]+$ && $CLCK_PROVIDER_SGEMM_FAST_MEMORY_LIMIT -gt 0 ]] ; then
            clck_sgemm_hbm_limit=$CLCK_PROVIDER_SGEMM_FAST_MEMORY_LIMIT
        fi
    fi
}

# prepare taskset command
clck_detect_taskset

case "${ARCH}" in
    "x86_64")
        export LD_LIBRARY_PATH=$(dirname $0)/../common/lib/intel64
        get_cpu_type
        # check the cpu type for Intel(R) Xeon Phi(TM) processor family,
        if [[ $clck_cpu_type =~ ^xeonphi* ]]; then
           # Following are matrix dimensions for SGEMM
           m=50006
           n=50006
           k=4320
           iterations=5
           clck_mu_rtype=1

           # prepare numactl command
           clck_detect_numactl

           # default the Fast Memory limit to 0 if not specified by the user
           clck_set_hbm_limit 0
           set_clck_iterations $iterations
           set_clck_m_n_k $m $n $k
           set_clck_kmp_affinity "granularity=fine,compact,1,0"
           set_clck_kmp_hw_subset "${num_physical_cores}c,${num_threads_per_core}t"

           export MKL_FAST_MEMORY_LIMIT=$clck_sgemm_hbm_limit
           exec $clck_numactl_bin $clck_taskset_bin $(dirname $0)/intel64/sgemm $clck_m_param $clck_n_param $clck_k_param $clck_sgemm_iterations 1
        else
           mem_percent=20
           iterations=5

           if [[ $clck_cpu_type == "xeon_scalable" ]]; then
             k=288
           else
             k=240
           fi

           set_clck_memory_usage $mem_percent $k

           memory_size 'KB'

           msq=$(( ($memory_size * 1024 * $clck_sgemm_memory_usage / 800) + ($clck_k_param * $clck_k_param) - $clck_k_param ))
           mi=`echo $msq | awk '{ printf("%d", sqrt($1)) }'`

           if (( $mi % 128 > 64 )); then
               m=$(( $mi + 128 - $mi % 128 ))
           else
               m=$(( $mi - $mi % 128 ))
           fi

           n=$m

           set_clck_iterations $iterations
           set_clck_m_n_k $m $n $clck_k_param
           set_clck_kmp_affinity "granularity=fine,compact,1,0"
           exec $clck_taskset_bin $(dirname $0)/intel64/sgemm $clck_m_param $clck_n_param $clck_k_param $clck_sgemm_iterations 1
        fi
        ;;

    *)
        echo "Error executing sgemm, node arch unknown. (${ARCH})"
        exit 1
        ;;
esac
