#!/bin/bash
# Copyright 2015-2019 Intel Corporation
#
# This software and the related documents are Intel copyrighted materials, and
# your use of them is governed by the express license under which they were
# provided to you (License). Unless the License provides otherwise, you may
# not use, modify, copy, publish, distribute, disclose, or transmit this
# software or the related documents without Intel's prior written permission.
#
# This software and the related documents are provided as is, with no express
# or implied warranties, other than those that are expressly stated in the
# License.

. $(dirname $0)/../../functions

memory_size 'MB'

# need at least 1500 MB
if [ $memory_size -lt 1500 ]; then
  echo "insufficent memory" >&2
  exit 1
fi

ARCH=`uname -m`

function get_task_set() {
    # number of supported cores and threads/core
    local taskset_threads=$(( $1*$2 ))
    taskset_threads=$(( taskset_threads-1 ))
    taskset_list="${taskset_threads}"
    taskset_list="0-${taskset_list}"
    if [[ "$CLCK_PROVIDER_STREAM_TASKSET" != "" ]]; then
        taskset_list="${CLCK_PROVIDER_STREAM_TASKSET}"
    fi
}

function clck_detect_taskset() {
    get_tool_presence "taskset"
    clck_taskset_bin=$tool_path
    if [[ "$CLCK_PROVIDER_STREAM_TASKSET_BINARY" != "" ]]; then
        clck_taskset_bin="${CLCK_PROVIDER_STREAM_TASKSET_BINARY}"
    fi
    # number of supported cores and threads/core
    num_logical_cores
    num_physical_cores
    num_threads_per_core
    get_task_set ${num_physical_cores} ${num_threads_per_core}
    if [[ $clck_taskset_bin == "CLCK_LOCATION_NOT_FOUND" ]]; then
        clck_taskset_bin="";
    else
        clck_taskset_bin="$clck_taskset_bin -c ${taskset_list}"
    fi
}

# prepare taskset command
clck_detect_taskset

if [[ "${CLCK_PROVIDER_STREAM_USE_AFFINITY}" == "yes" || ! -z  "${CLCK_PROVIDER_STREAM_KMP_AFFINITY}"  ]] ; then
  if [[ ! -z  "${CLCK_PROVIDER_STREAM_KMP_AFFINITY}" ]] ; then
   MY_KMP_AFFINITY=${CLCK_PROVIDER_STREAM_KMP_AFFINITY}
  else
   MY_KMP_AFFINITY=granularity=fine,compact,1,0
  fi
 else
   MY_KMP_AFFINITY=""
fi

if [[ "${CLCK_PROVIDER_STREAM_USE_PHYSICAL_CORES}" == "yes" ]] ; then
   MY_OMP_NUM_THREADS=${num_physical_cores}
else
   if [[ ! -z "${OMP_NUM_THREADS}" ]] ; then
       MY_OMP_NUM_THREADS=${OMP_NUM_THREADS}
   else
       MY_OMP_NUM_THREADS=${num_logical_cores}
   fi
fi

# prepare taskset command
case "${ARCH}" in
"x86_64")
    export LD_LIBRARY_PATH=$(dirname $0)/../common/lib/intel64
    echo KMP_AFFINITY=${MY_KMP_AFFINITY} OMP_NUM_THREADS=${MY_OMP_NUM_THREADS} exec $clck_taskset_bin $(dirname $0)/intel64/stream
    KMP_AFFINITY=${MY_KMP_AFFINITY} OMP_NUM_THREADS=${MY_OMP_NUM_THREADS} exec $clck_taskset_bin $(dirname $0)/intel64/stream
;;
*)
    echo "Error executing stream, node arch unknown. (${ARCH})" >&2
    exit 1
;;
esac

